// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.bulkeditor.command;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.Separator;
import org.eclipse.swt.SWT;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;
import org.eclipse.ui.menus.CommandContributionItem;
import org.eclipse.ui.menus.CommandContributionItemParameter;

import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorConstants;
import eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInputType;

/**
 * <p>
 * OpenBulkEditorContributionItem class.
 * </p>
 * 
 * @author p.ciardelli
 * @created 19.08.2009
 * @version 1.0
 */
public class OpenBulkEditorContributionItem extends CompoundContributionItem {

	/**
	 * <p>
	 * Constructor for OpenBulkEditorContributionItem.
	 * </p>
	 */
	public OpenBulkEditorContributionItem() {
	}

	/**
	 * <p>
	 * Constructor for OpenBulkEditorContributionItem.
	 * </p>
	 * 
	 * @param id
	 *            a {@link java.lang.String} object.
	 */
	public OpenBulkEditorContributionItem(String id) {
		super(id);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.actions.CompoundContributionItem#getContributionItems()
	 */
	/** {@inheritDoc} */
	@Override
	protected IContributionItem[] getContributionItems() {

		List<IContributionItem> contributionItems = new ArrayList<IContributionItem>();

		contributionItems.add(BulkEditorInputType.AGENT.createContributionItem());
		contributionItems.add(BulkEditorInputType.REFERENCE.createContributionItem());
		contributionItems.add(new Separator("test"));
		contributionItems.add(BulkEditorInputType.NAME.createContributionItem());
		contributionItems.add(BulkEditorInputType.NAME_RELATIONSHIP.createContributionItem());
		contributionItems.add(new Separator("test"));
		contributionItems.add(BulkEditorInputType.OCCURRENCE.createContributionItem());
		contributionItems.add(new Separator("test"));
		contributionItems.add(BulkEditorInputType.USER.createContributionItem());
		contributionItems.add(BulkEditorInputType.GROUP.createContributionItem());		

		return contributionItems
				.toArray(new IContributionItem[contributionItems.size()]);
	}

	/**
	 * @param key
	 * @param object
	 * @return
	 */
	private IContributionItem createContributionItem(String label,
			String inputType) {
		CommandContributionItemParameter parameter = new CommandContributionItemParameter(
				PlatformUI.getWorkbench().getActiveWorkbenchWindow(), null,
				IBulkEditorConstants.DYNAMIC_OPEN_MENU_ID, SWT.NONE);

		parameter.label = label;

		Map parameters = new HashMap();
		parameters.put(IBulkEditorConstants.INPUT_TYPE_PARAMETER_ID, inputType);
		parameter.parameters = parameters;

		return new CommandContributionItem(parameter);
	}
}
