// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.bulkeditor.input;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.UUID;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;

import eu.etaxonomy.cdm.api.service.ICommonService;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.strategy.merge.IMergable;
import eu.etaxonomy.cdm.strategy.merge.MergeException;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditorQuery;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditorUtil;
import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorSortProvider;
import eu.etaxonomy.taxeditor.bulkeditor.input.sortprovider.CdmBaseSortProvider;
import eu.etaxonomy.taxeditor.bulkeditor.input.sortprovider.TitleCacheComparator;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>Abstract AbstractBulkEditorInput class.</p>
 *
 * @author p.ciardelli
 * @created 25.06.2009
 * @version 1.0
 * @param <T>
 */
public abstract class AbstractBulkEditorInput<T extends ICdmBase> implements IEditorInput, IEntityPersistenceService<T> {

	/**
	 * 
	 */
	private static final long serialVersionUID = 416414530232743735L;

	private UUID entityUuid;

	private List<T> model;
	
	private IEntityCreator<T> entityCreator;
	
	/**
	 * <p>NewInstance</p>
	 *
	 * @param inputType a {@link eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInputTypeValues.BulkEditorInputType} object.
	 * @return a {@link eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput} object.
	 */
	static public AbstractBulkEditorInput NewInstance(BulkEditorInputType inputType) {
		return BulkEditorInputType.getInput(inputType);
	}
	
	/**
	 * <p>NewInstance</p>
	 *
	 * @param entity a {@link eu.etaxonomy.cdm.model.common.IdentifiableEntity} object.
	 * @return a {@link eu.etaxonomy.taxeditor.bulkeditor.input.AbstractBulkEditorInput} object.
	 */
	public static AbstractBulkEditorInput NewInstance(IdentifiableEntity entity) {
		
		
		BulkEditorInputType inputType = BulkEditorInputType.getByType(entity.getClass());
		
		AbstractBulkEditorInput editorInput = NewInstance(inputType);
		
		editorInput.setEntityUuid(entity.getUuid());
		
		return editorInput;
	}
	
	/**
	 * <p>listEntities</p>
	 *
	 * @param configurator a {@link eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator} object.
	 * @return a {@link java.util.List} object.
	 */
	protected abstract List<T> listEntities(IIdentifiableEntityServiceConfigurator configurator);
		
	/**
	 * <p>loadEntity</p>
	 *
	 * @param entityUuid a {@link java.util.UUID} object.
	 * @return a T object.
	 */
	protected abstract T loadEntity(UUID entityUuid);

	private void setEntityUuid(UUID entityUuid){
		this.entityUuid = entityUuid;
	}
	
	/**
	 * <p>Getter for the field <code>entityUuid</code>.</p>
	 *
	 * @return a {@link java.util.UUID} object.
	 */
	public UUID getEntityUuid() {
		return entityUuid;
	}

			
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#exists()
	 */
	/**
	 * <p>exists</p>
	 *
	 * @return a boolean.
	 */
	public boolean exists() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getImageDescriptor()
	 */
	/**
	 * <p>getImageDescriptor</p>
	 *
	 * @return a {@link org.eclipse.jface.resource.ImageDescriptor} object.
	 */
	public ImageDescriptor getImageDescriptor() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IEditorInput#getPersistable()
	 */
	/**
	 * <p>getPersistable</p>
	 *
	 * @return a {@link org.eclipse.ui.IPersistableElement} object.
	 */
	public IPersistableElement getPersistable() {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	public Object getAdapter(Class adapter) {
		return null;
	}

	/**
	 * <p>Setter for the field <code>query</code>.</p>
	 *
	 * @param bulkEditorQuery a {@link eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorQuery} object.
	 */
	public void performSearch(final BulkEditorQuery bulkEditorQuery) {
		
		List<T> entityList = new ArrayList<T>();
		
		if(getEntityUuid() != null){
			
			T entity = loadEntity(getEntityUuid());
			entityList.add(entity);
			model = entityList;
		}
		else if(bulkEditorQuery != null){
			
			String queryString = bulkEditorQuery.getSearchString();
			IIdentifiableEntityServiceConfigurator configurator = bulkEditorQuery.getSearchConfigurator();
			Comparator queryComparator = (bulkEditorQuery.getComparator() != null) ? bulkEditorQuery.getComparator() : new TitleCacheComparator();
			
			entityList = listEntities(configurator);
			
			Collections.sort(entityList, queryComparator);			
			
		}
		
		model = entityList;
	}
	
	/**
	 * <p>isMergingEnabled</p>
	 *
	 * @return a boolean.
	 */
	public boolean isMergingEnabled() {
		return false;
	}

	/**
	 * <p>isMarkerTypeEditingEnabled</p>
	 *
	 * @param markerType a {@link eu.etaxonomy.cdm.model.common.MarkerType} object.
	 * @return a boolean.
	 */
	public boolean isMarkerTypeEditingEnabled(MarkerType markerType) {
		return false;
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#merge(eu.etaxonomy.cdm.model.common.IdentifiableEntity, eu.etaxonomy.cdm.model.common.IdentifiableEntity)
	 */
	/** {@inheritDoc} */
	public boolean merge(T entity, T mergeTarget) {
		if (entity instanceof IMergable) {
			try {
				CdmStore.getService(ICommonService.class).merge((IMergable)mergeTarget, (IMergable)entity, null);
			} catch (MergeException e) {
				BulkEditorUtil.error(getClass(), e);
			}
		}
		return true;
	};
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityPersistenceService#create(eu.etaxonomy.cdm.model.common.IdentifiableEntity)
	 */
	/** {@inheritDoc} */
	public boolean create(T entity) {
		return save(entity);		
	};
	
	public IEntityCreator<T> getEntityCreator(){
		if(entityCreator == null){
			entityCreator = createEntityCreator();
		}
		return entityCreator;
	}

	/**
	 * @return
	 */
	protected abstract IEntityCreator<T> createEntityCreator();

	/**
	 * The default implementation returns an empty list of sort providers.
	 * @return
	 */
	public List<IBulkEditorSortProvider<T>> getSortProviders(){
		List<IBulkEditorSortProvider<T>> sortProviders = new ArrayList<IBulkEditorSortProvider<T>>();
		
		sortProviders.add(new CdmBaseSortProvider<T>());
		
		return sortProviders;
	}

	/**
	 * @param entity
	 * @return
	 */
	public String getTypeText(Object entity){
		return entity.getClass().getSimpleName();
	}

	/**
	 * @param entity
	 * @return
	 */
	public String getText(T entity) {
		if(entity instanceof IdentifiableEntity){
			IdentifiableEntity identifiableEntity = (IdentifiableEntity) HibernateProxyHelper.deproxy(entity);
			
			return identifiableEntity.getTitleCache();
		}
		
		return "No text. Implement in subclass";
	}
	
	/**
	 * @return
	 */
	public List<?> getModel() {
		return model;
	}


}
