/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Display;

/**
 * <p>SimpleSelectionProvider class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 16.05.2008
 * @version 1.0
 */
public class SimpleSelectionProvider implements ISelectionProvider {

	private Set<ISelectionChangedListener> selectionChangedListeners = new HashSet<ISelectionChangedListener>();
	private ISelection selection;
	private Job job;
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#addSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	/** {@inheritDoc} */
	public void addSelectionChangedListener(
			ISelectionChangedListener listener) {
		selectionChangedListeners.add(listener);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#getSelection()
	 */
	/**
	 * <p>Getter for the field <code>selection</code>.</p>
	 *
	 * @return a {@link org.eclipse.jface.viewers.ISelection} object.
	 */
	public ISelection getSelection() {
		if (selection != null){
			return selection;
		}
		// TODO we have to return an empty selection to avoid NPE
		// I don't really understand why this was not the case before
		return new StructuredSelection();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#removeSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
	 */
	/** {@inheritDoc} */
	public void removeSelectionChangedListener(
			ISelectionChangedListener listener) {
		selectionChangedListeners.remove(listener);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ISelectionProvider#setSelection(org.eclipse.jface.viewers.ISelection)
	 */
	/** {@inheritDoc} */
	public void setSelection(ISelection selection) {
		this.selection = selection;
		
		// cancel previous selection setting
		if(job != null){
			if(job.getState() != Job.NONE){
				job.cancel();
			}
			job = null;
		}
		
		if(job == null){
			final SelectionChangedEvent selectionChangedEvent = new SelectionChangedEvent(this, selection);
			final Display display = Display.getCurrent();
			job = new Job("Setting Selection"){
	
				@Override
				protected IStatus run(IProgressMonitor monitor) {
					monitor.beginTask("Setting Selection", 10);
					
					if(!monitor.isCanceled()){
						display.asyncExec(new Runnable() {
					 
							public void run() {
								SimpleSelectionProvider.this.fireSelectionChanged(selectionChangedEvent);
							}
						});
					}
					
					return Status.OK_STATUS;
				}
				
			};
			
			job.setPriority(Job.DECORATE);
			job.schedule();
		}
		
	}

	private void fireSelectionChanged(final SelectionChangedEvent event) {
		
		for (final ISelectionChangedListener listener : selectionChangedListeners) {
			SafeRunnable.run(new SafeRunnable() {
				public void run() {
					listener.selectionChanged(event);
				}
			});
		}
	}
}
