// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.descriptive;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.MoveDescriptionElementsOperation;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author n.hoffmann
 * @created Feb 8, 2011
 * @version 1.0
 */
public class DescriptionElementDropAdapter extends ViewerDropAdapter {

	/**
	 * @param viewer
	 */
	public DescriptionElementDropAdapter(Viewer viewer) {
		super(viewer);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ViewerDropAdapter#performDrop(java.lang.Object)
	 */
	@Override
	public boolean performDrop(Object data) {
		TaxonDescription target = (TaxonDescription) getCurrentTarget();
		Object[] droppedElements = (Object[]) data;
		
		Collection<DescriptionElementBase> descriptionElements = new ArrayList<DescriptionElementBase>();
		
		boolean isCopy = getCurrentOperation() == DND.DROP_COPY ? true : false;
		
		// cannot drop a feature node onto itself
		for (Object droppedElement : droppedElements) {			
			if (droppedElement == null){
				StoreUtil.warningDialog("Operation not supported yet", this, "We are currently unable to drag and drop a newly created element. Please save the editor to make this work.");
				return false;
			}
			if(! (droppedElement instanceof DescriptionElementBase)){
				return false;
			}else{
				DescriptionElementBase descriptionElement = (DescriptionElementBase) droppedElement;
				
				if (descriptionElement.getInDescription().equals(target)) {
					return false;
				}
				
				descriptionElements.add(descriptionElement);
			}			
		}
		
		AbstractPostOperation operation = new MoveDescriptionElementsOperation("Move Descriptions", EditorUtil.getUndoContext(), target, descriptionElements, isCopy, null);
		
		EditorUtil.executeOperation(operation);
		
		return true;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ViewerDropAdapter#validateDrop(java.lang.Object, int, org.eclipse.swt.dnd.TransferData)
	 */
	@Override
	public boolean validateDrop(Object target, int operation,
			TransferData transferData) {
		boolean transferDataIsSupported = DescriptionElementTransfer.getInstance().isSupportedType(
				transferData);
		System.out.println(target);
		return target instanceof TaxonDescription && transferDataIsSupported;
	}

}
