// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.descriptive.handler;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.api.service.IDescriptionService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptiveViewPart;
import eu.etaxonomy.taxeditor.editor.view.descriptive.operation.MoveDescriptionElementsOperation;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TaxonBaseSelectionDialog;


/**
 * @author n.hoffmann
 * @created Feb 8, 2011
 * @version 1.0
 */
public class MoveDescriptionElementsHandler extends AbstractHandler {

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		
//		ConversationHolder conversation = CdmStore.createConversation();
		
		ISelection selection = HandlerUtil.getCurrentSelection(event);
		
		if(selection instanceof IStructuredSelection){

			IStructuredSelection structuredSelection = (IStructuredSelection) selection;
			
			List<DescriptionElementBase> elements = new ArrayList<DescriptionElementBase>();
			
			for(Object element : structuredSelection.toArray()){
				if(element instanceof DescriptionElementBase){
					UUID uuid = ((DescriptionElementBase) element).getUuid();
					
					elements.add(CdmStore.getService(IDescriptionService.class).loadDescriptionElement(uuid, null));
				}
			}
			
			Taxon targetTaxon = TaxonBaseSelectionDialog.selectTaxon(HandlerUtil.getActiveShell(event), EditorUtil.getActiveMultiPageTaxonEditor().getConversationHolder(), null);
						
			TaxonDescription targetDescription = TaxonDescription.NewInstance(targetTaxon);
			targetDescription.setTitleCache(String.format("Copied from %s", EditorUtil.getActiveMultiPageTaxonEditor().getTaxon()), true);
			
			try {
				AbstractPostOperation operation = new MoveDescriptionElementsOperation(
						event.getCommand().getName(), EditorUtil.getUndoContext(), 
						targetDescription, elements, false, (IPostOperationEnabled) EditorUtil.getView(DescriptiveViewPart.ID, true));
				EditorUtil.executeOperation(operation);
//				conversation.commit(true);
				CdmStore.getService(ITaxonService.class).saveOrUpdate(targetTaxon);
				
			} catch (NotDefinedException e) {
				EditorUtil.error(getClass(), e);
			}
		}
		
		return null;
	}

}
