/**
 * Copyright (C) 2011 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.editor.view.uses;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.FeatureTree;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.UsageTermCollection;
import eu.etaxonomy.taxeditor.editor.view.descriptive.DescriptiveContentProvider;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainerTree;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.TermStore;

/**
 * The class provides the required content to the Uses View
 * @author a.theys	
 * @created mar 13, 2012
 * @version 1.0
 */
public class UsesContentProvider extends DescriptiveContentProvider implements ITreeContentProvider {

	Set<MarkerType> markerTypes = new HashSet<MarkerType>();
		
	/**
	 * <p>Constructor for DescriptiveContentProvider.</p>
	 *
	 * @param featureNodeContainerCache a {@link java.util.Map} object.
	 */
	public UsesContentProvider(Map<TaxonDescription, FeatureNodeContainerTree> featureNodeContainerCache) {
		super(featureNodeContainerCache);
		this.featureNodeContainerCache = featureNodeContainerCache;
	}

	/**
	 * Get all descriptions associated with the given TaxonEditorInput
	 * 
	 * @param parentElement
	 * @return
	 */
	@Override
	protected List<DescriptionBase> getDescriptions(TaxonEditorInput parentElement) {
		this.markerTypes.addAll(CdmStore.getTermManager().getPreferredTerms(MarkerType.class));
		Taxon taxon = parentElement.getTaxon();
		List<DescriptionBase> descriptions = new ArrayList<DescriptionBase>();
		
		for(DescriptionBase description : taxon.getDescriptions()){
			if(! description.isImageGallery()){
				MarkerType useMarkertype = (MarkerType) CdmStore.getService(ITermService.class).find(UsageTermCollection.uuidUseMarkerType);
				Set<Marker> descriptionMarkers = description.getMarkers();
				if(descriptionMarkers != null) {
					for (Marker marker: descriptionMarkers) {
						if(marker.getMarkerType().equals(useMarkertype)) {
							descriptions.add(description);
						}
					}
				}
			}
		}			
		return descriptions;
	}

}
