/**
* Copyright (C) 2011 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.uses;

import java.util.Arrays;
import java.util.List;

import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;
import org.eclipse.jface.viewers.StyledString;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.Modifier;
import eu.etaxonomy.cdm.model.description.StateData;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.taxeditor.model.FeatureNodeContainer;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.editor.UsageTermCollection;

/**
 * UsesLabelProvider Class
 * @author a.theys	
 * @created mar 13, 2012
 * @version 1.0
 */
public class UsesLabelProvider extends ColumnLabelProvider implements
		IStyledLabelProvider {
	
	/** {@inheritDoc} */
	public String getText(Object element) {
		String text = "";

		if (element instanceof DescriptionBase) {
			text = ((DescriptionBase) element).getTitleCache();
			if (text == null || text.length() == 0) {
				text = "Use: No label provided";
			}
			else {
				text = "Use: " + text;
			}
		}
		else if (element instanceof CategoricalData) {
			if (!((CategoricalData) element).getStates().isEmpty()) {
				boolean isUseCategoryAbsent = true;
				for (StateData statedata : ((CategoricalData) element).getStates()) {
					if(statedata.getState() != null && statedata.getModifyingText().get(CdmStore.getDefaultLanguage()) !=null && statedata.getModifyingText().get(CdmStore.getDefaultLanguage()).getText().equals(UsageTermCollection.useCategoryVocabularyLabel)) {
						text = text + statedata.getState().getTitleCache() + ";";
						isUseCategoryAbsent = false;
					}
				}
				if(isUseCategoryAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + ";";
				}
				
				boolean isUseSubCategoryAbsent = true;
				for (StateData statedata : ((CategoricalData) element).getStates()) {
					if(statedata.getState() != null && statedata.getModifyingText().get(CdmStore.getDefaultLanguage()) !=null && statedata.getModifyingText().get(CdmStore.getDefaultLanguage()).getText().equals(UsageTermCollection.useSubCategoryVocabularyLabel)) {
						text = text + statedata.getState().getTitleCache() + ";";
						isUseSubCategoryAbsent = false;
					}
				}
				if(isUseSubCategoryAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + ";";
				}
			}
			else {
				text = text + UsageTermCollection.notAvailableLabel + ";" + UsageTermCollection.notAvailableLabel + ";";
			}
			if (!((CategoricalData) element).getModifiers().isEmpty()) {
				for (Modifier modifier: ((CategoricalData) element).getModifiers()){
					modifier.getPartOf();
					modifier.getVocabulary();
				}
				
				boolean isPlantPartAbsent = true;
				for (Modifier modifier: ((CategoricalData) element).getModifiers()){
					if(GetVocabularyType(modifier, UsageTermCollection.plantPartLabel)) {
						text = text +  modifier.getTitleCache()+ ";";
						isPlantPartAbsent = false;
					}
					
				}
				if (isPlantPartAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + ";";
				}
				
				boolean isHumanGroupAbsent = true;
				for (Modifier modifier: ((CategoricalData) element).getModifiers()){
					if(GetVocabularyType(modifier, UsageTermCollection.humanGroupLabel)) {
						text = text +  modifier.getTitleCache()+ ";";
						isHumanGroupAbsent = false;
					}
				}
				if (isHumanGroupAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + ";";
				}
				
				boolean isEthnicGroupAbsent = true;
				for (Modifier modifier: ((CategoricalData) element).getModifiers()){
					if(GetVocabularyType(modifier, UsageTermCollection.ethnicGroupLabel)) {
						text = text +  modifier.getTitleCache()+ ";";
						isEthnicGroupAbsent = false;
					}
				}
				if (isEthnicGroupAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + ";";
				}
				
				boolean isCountryAbsent = true;
				for (Modifier modifier: ((CategoricalData) element).getModifiers()){
					if(GetVocabularyType(modifier, UsageTermCollection.countryLabel)) {
						text = text +  modifier.getTitleCache()+ ";";
						isCountryAbsent = false;
					}
				}
				if (isCountryAbsent) {
					text = text + UsageTermCollection.notAvailableLabel + ";";
				}
			}
			else {
				text = text + UsageTermCollection.notAvailableLabel + ";" + UsageTermCollection.notAvailableLabel + ";" + UsageTermCollection.notAvailableLabel + ";" + UsageTermCollection.notAvailableLabel + ";";
			}
		
			if (text == null || text.length() == 0) {
				text = "No data provided";
			}
			else {
				String[] textElements = text.split(";");
				String dataportalString = textElements[0] + ";" + textElements[1] + ";" + textElements[5] + ";" + textElements[2] + ";" + textElements[3] + ";" + textElements[4] + ";" ;  
				
				((CategoricalData)element).putModifyingText(CdmStore.getDefaultLanguage(), dataportalString);
			}
			return text;
		}
		
		else if (element instanceof TextData) {
			List<Language> languages = Arrays.asList(new Language[]{CdmStore.getDefaultLanguage()});
			LanguageString languageString = ((TextData) element).getPreferredLanguageString(languages);
			text = languageString != null ? languageString.getText() : null;
			if (text == null || text.length() == 0) {
				text = "No data provided";
			}
		}
		
		else if (element instanceof FeatureNodeContainer){
			text = ((FeatureNodeContainer)element).getFeature().getTitleCache();
		}

		return  text;
	}

	/** Function dertmining which vocabulary is the term part of 
	 *  
	 * @param term
	 * @param vocabularyExpected
	 * @return
	 */
	private boolean GetVocabularyType(Modifier term, String vocabularyExpected) {
		if ((term.getPartOf() != null)  && (term.getPartOf().getTitleCache().equals(vocabularyExpected))) {
			return true;
		}
		else if ((term.getVocabulary() != null) && (term.getPartOf() == null) && (term.getVocabulary().getTitleCache().equals(vocabularyExpected))) {
			return true;
		}
		else if ((vocabularyExpected.equals(UsageTermCollection.ethnicGroupLabel))  && ((term.getPartOf() != null))) {
			return true;
		}
	return false;
	}
	
	/** {@inheritDoc} */
	@Override
	public StyledString getStyledText(Object element) {
		return new StyledString(this.getText(element),
				StyledString.QUALIFIER_STYLER);
	}
	
	

}
