// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.navigator;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider;
import org.eclipse.jface.viewers.StyledString.Styler;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.TextStyle;
import org.eclipse.ui.navigator.IDescriptionProvider;
import org.hibernate.LazyInitializationException;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.IIdentifiableEntity;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.persistence.hibernate.permission.Operation;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.security.RequiredPermissions;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>TaxonNodeLabelProvider class.</p>
 *
 * @author p.ciardelli
 * @created 04.06.2009
 * @version 1.0
 */
public class TaxonNodeLabelProvider extends ColumnLabelProvider 
		implements ILabelProvider, IDescriptionProvider, IStyledLabelProvider {
	

	private Styler notGrantedStyler = null;
	
	/** {@inheritDoc} */
	@Override
	public Image getImage(Object element) {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getText(Object element) {
		if (element instanceof TaxonNode){
			TaxonNode taxonNode = (TaxonNode) HibernateProxyHelper.deproxy(element);
			
			try{
				Taxon taxon = (Taxon) HibernateProxyHelper.deproxy(taxonNode.getTaxon());
				if(taxon == null){
					NavigationUtil.error(getClass(), "The taxon node does not have a taxon attached to it. Please check your import. TaxonNode: " + element, null);
//					throw new RuntimeException("The data your are trying to view is corrupt. Please check your import.");
				}else{
					try{					
						return taxon.getName() != null ? ((IIdentifiableEntity) HibernateProxyHelper.deproxy(taxon.getName())).getTitleCache() : new String();
					}catch(Exception e){
						NavigationUtil.error(getClass(), e);
					}
				}
			}catch (LazyInitializationException e){
				NavigationUtil.error(getClass(), e);
			}
			
		}
		return new String();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.IDescriptionProvider#getDescription(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public String getDescription(Object anElement) {
		if (anElement instanceof TaxonNode) {
			Taxon data = ((TaxonNode) anElement).getTaxon();
			return "Taxon: " + data.getTitleCache(); //$NON-NLS-1$
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider.IStyledLabelProvider#getStyledText(java.lang.Object)
	 */
	/** {@inheritDoc} */
	public StyledString getStyledText(Object element) {
		// determine style base on  user grants
		Styler styler = null;
		if(!CdmStore.currentAuthentiationHasPermission((CdmBase)element, RequiredPermissions.TAXONNODE_EDIT)){
			styler = getNotGrantedStyler();
		}
		return new StyledString(getText(element), styler);
		
	}
	
	/**
	 * @return
	 */
	private Styler getNotGrantedStyler() {
		if (notGrantedStyler  == null) {
			notGrantedStyler = new Styler() {
				@Override
				public void applyStyles(TextStyle textStyle) {
					textStyle.underline = false;
					textStyle.foreground = StoreUtil.getColor(Resources.COLOR_TEXT_DISABLED);
				}
			};
		}
		return notGrantedStyler;
	}
}
