// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.printpublisher.wizard;

import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.jdom.Element;

import eu.etaxonomy.cdm.print.IXMLEntityFactory;
import eu.etaxonomy.cdm.print.PublishConfigurator;
import eu.etaxonomy.cdm.print.XMLHelper;
import eu.etaxonomy.cdm.print.XMLHelper.EntityType;
import eu.etaxonomy.taxeditor.model.CdmProgressMonitorAdapter;
import eu.etaxonomy.taxeditor.printpublisher.PrintUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>SelectServiceWizardPage class.</p>
 *
 * @author n.hoffmann
 * @created Apr 6, 2010
 * @version 1.0
 */
public class SelectServiceWizardPage extends AbstractPublishWizardPage {

	private Composite composite;
	private Button button_local;
	private Button button_remote;
	private Text text_serviceUrl;
	
	private TreeViewer treeViewer;

	/**
	 * <p>Constructor for SelectServiceWizardPage.</p>
	 *
	 * @param pageName a {@link java.lang.String} object.
	 */
	protected SelectServiceWizardPage(String pageName) {
		super(pageName);
		setTitle("Select a Service");
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	/** {@inheritDoc} */
	public void createControl(Composite parent) {
		
		composite = new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout());
		
		RadioSelectionListener listener = new RadioSelectionListener();
		
		Group radioGroup = new Group(composite, SWT.SHADOW_ETCHED_IN);
		radioGroup.setLayout(new GridLayout());
		
		button_local = new Button(radioGroup, SWT.RADIO);
				
		button_local.setText("Local (By selecting this option the database you are currently " +
				"connected to will be used to gather data.)");
		
		button_local.addSelectionListener(listener);
		
		
						
		button_remote = new Button(radioGroup, SWT.RADIO);
		button_remote.setText("Remote");
		button_remote.addSelectionListener(listener);
		
		text_serviceUrl = new Text(radioGroup, SWT.BORDER);
		text_serviceUrl.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		text_serviceUrl.setText("http://");	
		text_serviceUrl.addModifyListener(new ModifyListener() {
			
			public void modifyText(ModifyEvent e) {
				String text = text_serviceUrl.getText();
				
				if(!text.endsWith("/")){
					SelectServiceWizardPage.this.setErrorMessage("Webservice URL has to end with \"/\"");
					setPageComplete(false);
					return;
				}
				
				URL url = null;
				try {
					url = new URL(text);
				} catch (MalformedURLException e1) {
					SelectServiceWizardPage.this.setErrorMessage("Webservice URL is malformed.");
					setPageComplete(false);
					return;
				}
				
				getConfigurator().setWebserviceUrl(url);
				
				SelectServiceWizardPage.this.setErrorMessage(null);
					
			}
		});
		
		treeViewer = new TreeViewer(composite);
		
		treeViewer.setContentProvider(new ContentProvider());
		treeViewer.setLabelProvider(new LabelProvider());
		
		treeViewer.addSelectionChangedListener(new SelectionChangedListener());
		
		treeViewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		
		if(CdmStore.isActive()){
			enableLocal();
		}else{
			enableRemote();
			button_local.setEnabled(false);
		}
		
		setControl(composite);
		
	}
	
	private class RadioSelectionListener extends SelectionAdapter{
		@Override
		public void widgetSelected(SelectionEvent e) {
			if(button_local.getSelection()){
				enableLocal();
			}else if(button_remote.getSelection()){
				enableRemote();
			}
		}
	}
	
	private void enableRemote() {
		button_local.setSelection(false);
		button_remote.setSelection(true);
		
		text_serviceUrl.setEnabled(true);
		
		setConfigurator(PublishConfigurator.NewRemoteInstance());
		getConfigurator().addOutputModule(getOutputModule());
	}

	private void enableLocal() {
		button_remote.setSelection(false);
		button_local.setSelection(true);
		
		text_serviceUrl.setEnabled(false);
		
		setConfigurator(PublishConfigurator.NewLocalInstance(CdmStore.getCurrentApplicationConfiguration()));
		getConfigurator().addOutputModule(getOutputModule());
		refresh();
	}
	
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.wizard.WizardPage#canFlipToNextPage()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean canFlipToNextPage() {	
		return isPageComplete();
	}
	
	/**
	 * <p>refresh</p>
	 */
	public void refresh(){
		
		if(getConfigurator() != null){
		
			IRunnableWithProgress runnable = new IRunnableWithProgress(){

				@Override
				public void run(IProgressMonitor monitor) {
					monitor.beginTask("Loading classifications", IProgressMonitor.UNKNOWN);
					IXMLEntityFactory factory = getConfigurator().getFactory();
					final List<Element> classifications = factory.getClassifications();
					
					Display.getDefault().asyncExec(new Runnable(){

						@Override
						public void run() {
							treeViewer.setInput(classifications);
						}
						
					});
					monitor.done();
				}
				
			};
			try {
				getContainer().run(true, false, runnable);
			} catch (InvocationTargetException e) {
				StoreUtil.error(this.getClass(), e);
			} catch (InterruptedException e) {
				StoreUtil.error(this.getClass(), e);
			}
		}
	}
		
	/*
	 * (non-Javadoc)
	 * @see org.eclipse.jface.wizard.WizardPage#isPageComplete()
	 */
	/** {@inheritDoc} */
	@Override
	public boolean isPageComplete() {
		boolean complete = true;
		if(getConfigurator().isLocal()){
			complete &= true;
		}else if(getConfigurator().isRemote() 
				&& getConfigurator().getWebserviceUrl() != null 
				&& getConfigurator().getWebserviceUrl().toString().endsWith("/")){
			complete &= true;
		}else{
			return false;
		}
		
		List<Element> selectedTaxonNodes = getConfigurator().getSelectedTaxonNodeElements();
		
		complete &= !selectedTaxonNodes.isEmpty();
		
		return complete;
	}
	
	private class SelectionChangedListener implements ISelectionChangedListener {

		public void selectionChanged(SelectionChangedEvent event) {
			StructuredSelection selection = (StructuredSelection) treeViewer.getSelection();
			
			List<Element> selectedElements = selection.toList();
			if(selectedElements.size() > 0){
				getConfigurator().setSelectedTaxonNodeElements(selectedElements);
				setPageComplete(true);
			}
		}
		
	}
	
	private class ContentProvider implements ITreeContentProvider{

		public Object[] getChildren(Object parentElement) {
			if(parentElement instanceof List){
				return ((List)parentElement).toArray();
			}
			else if(parentElement instanceof Element){
				Element element = (Element) parentElement;
				
				IXMLEntityFactory factory = getConfigurator().getFactory();
				
				return factory != null ? factory.getChildNodes(element).toArray() : new Object[]{};
				
			}
			
			return new Object[]{};
		}

		public Object getParent(Object element) {
			return null;
		}

		public boolean hasChildren(Object element) {
			return getChildren(element).length > 0;
		}

		public Object[] getElements(Object inputElement) {
			return getChildren(inputElement);
		}

		public void dispose() {
			
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			
		}
		
	}
	
	private class LabelProvider  extends ColumnLabelProvider
		implements ILabelProvider{
		
		@Override
		public String getText(Object element) {
			if(element instanceof Element){
				Element xmlElement = (Element) element;
				EntityType entityType = XMLHelper.getEntityType(xmlElement);
				if(EntityType.TAXON_NODE.equals(entityType)){
					xmlElement = getConfigurator().getFactory().getTaxonForTaxonNode(xmlElement);
				}				
				return XMLHelper.getTitleCache(xmlElement);
			}
			return "no title cache";
		}
		
	}
	
}
