// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io.wizard;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.IWizardContainer;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IImportWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;

import eu.etaxonomy.cdm.io.common.IImportConfigurator;
import eu.etaxonomy.cdm.io.common.IIoConfigurator;

/**
 * <p>Abstract AbstractImportWizard class.</p>
 *
 * @author n.hoffmann
 * @created 24.06.2009
 * @version 1.0
 */
public abstract class AbstractImportWizard<CONFIG extends IIoConfigurator> extends Wizard implements IImportWizard {
	
	protected final String CONFIGURATION_PAGE = "CONFIGURATION_PAGE"; 
	
	private GenericConfiguratorWizardPage pageConfiguration;
	private NomenclaturalCodeWizardPage pageNomenclaturalCode;

	private IWorkbench workbench;

	private IStructuredSelection selection;
	

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	/** {@inheritDoc} */
	@Override
	public void addPages() {
		super.addPages();
		
		pageConfiguration = GenericConfiguratorWizardPage.Import(CONFIGURATION_PAGE, getConfigurator());
		this.addPage(pageConfiguration);
		
		pageNomenclaturalCode = new NomenclaturalCodeWizardPage((IImportConfigurator)getConfigurator());
		this.addPage(pageNomenclaturalCode);
		
		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#setContainer(org.eclipse.jface.wizard.IWizardContainer)
	 */
	@Override
	public void setContainer(IWizardContainer wizardContainer) {
		super.setContainer(wizardContainer);
		checkForUnsavedEditors();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	@Override
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.setWorkbench(workbench);
		this.setSelection(selection);
	}
	
	protected void checkForUnsavedEditors(){
		IWorkbenchPage activePage = getWorkbench().getActiveWorkbenchWindow().getActivePage();
		
		if(activePage != null){
			for (IEditorReference reference : activePage.getEditorReferences()) {
				IEditorPart editor = reference.getEditor(false);
				if(editor.isDirty()){
					
					MessageDialog.open(MessageDialog.WARNING, getShell(), "Unsaved editors", "Please save all open editors before executing " +
							"an import operation. The import operation will be cancelled", SWT.NONE);
					
					performCancel();
					if(getContainer() instanceof WizardDialog){
						((WizardDialog)getContainer()).close();
					}
					
				}
			}
		}
	}
	
	/**
	 * <p>getConfigurator</p>
	 *
	 * @return a CONFIG object.
	 */
	public abstract CONFIG getConfigurator();

	/**
	 * @param selection the selection to set
	 */
	public void setSelection(IStructuredSelection selection) {
		this.selection = selection;
	}

	/**
	 * @return the selection
	 */
	public IStructuredSelection getSelection() {
		return selection;
	}

	/**
	 * @param workbench the workbench to set
	 */
	public void setWorkbench(IWorkbench workbench) {
		this.workbench = workbench;
	}

	/**
	 * @return the workbench
	 */
	public IWorkbench getWorkbench() {
		return workbench;
	}
}
