// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.TransferData;

import eu.etaxonomy.cdm.model.common.ICdmBase;

/**
 * @author n.hoffmann
 * @created Feb 8, 2011
 * @version 1.0
 */
public abstract class CdmObjectTransfer<T extends ICdmBase> extends ByteArrayTransfer {
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.dnd.ByteArrayTransfer#javaToNative(java.lang.Object, org.eclipse.swt.dnd.TransferData)
	 */
	@Override
	protected void javaToNative(Object object, TransferData transferData) {
		byte[] bytes = toByteArray((T[]) object);
		if (bytes != null)
			super.javaToNative(bytes, transferData);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.dnd.ByteArrayTransfer#nativeToJava(org.eclipse.swt.dnd.TransferData)
	 */
	@Override
	protected Object nativeToJava(TransferData transferData) {
		byte[] bytes = (byte[]) super.nativeToJava(transferData);
		return fromByteArray(bytes);
	}
	
	protected byte[] toByteArray(T[] elements) {
		ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
		DataOutputStream out = new DataOutputStream(byteOut);

		byte[] bytes = null;

		try {
			
			for(T element : elements){
				writeElementUuid(element, out);
			}
			out.close();
			bytes = byteOut.toByteArray();
		} catch (IOException e) {
			// when in doubt send nothing
		}
		return bytes;
	}
	
	private void writeElementUuid(T element, DataOutputStream out) throws IOException {
		out.writeUTF(element.getUuid().toString());		
	}

	protected Object fromByteArray(byte[] bytes) {
		DataInputStream in = new DataInputStream(
				new ByteArrayInputStream(bytes));

		try {
			List<T> elements = new ArrayList<T>();
			
			try{
				while(true){
					UUID uuid = readElementUuid(in);
					elements.add(loadElement(uuid));
				}
			}catch(EOFException e){
				return elements.toArray();
			}
		} catch (IOException e) {
			return null;
		}
	}

	public abstract T loadElement(UUID uuid);
	
	/**
	 * @param in
	 * @return
	 * @throws IOException 
	 */
	public UUID readElementUuid(DataInputStream in) throws IOException{
		return UUID.fromString(in.readUTF());
	}
}
