/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.DeviceResourceException;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;

import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * <p>ImageResources class.</p>
 *
 * @author n.hoffmann
 * @created 24.03.2009
 * @version 1.0
 */
public class ImageResources {
	
	private static ImageResources instance;
	
	/**
	 * <p>Getter for the field <code>instance</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.model.ImageResources} object.
	 */
	public static ImageResources getInstance(){
		if(instance == null){
			instance = new ImageResources();
		}
		return instance;
	}
	
	
	/* ***************************************************************************************
	ICONS
	************************************************************************************** */
	/** Constant <code>ADD_ICON="add_icon"</code> */
	public static final String ADD_ICON = "add_icon";
	/** Constant <code>TRASH_ICON="trash_icon"</code> */
	public static final String TRASH_ICON = "trash_icon";
	/** Constant <code>BROWSE_ICON="browse_icon"</code> */
	public static final String BROWSE_ICON = "browse_icon";
	/** Constant <code>EDIT_ICON="edit_icon"</code> */
	public static final String EDIT_ICON = "edit_icon";
	/** Constant <code>EDIT_LOGO_ICON="edit_logo_icon"</code> */
	public static final String EDIT_LOGO_ICON = "edit_logo_icon";
	/** Constant <code>WARNING_ICON="warning_icon"</code> */
	public static final String WARNING_ICON = "warning_icon";
	/** Constant <code>BLACK_SQUARE_ICON="black_square_icon"</code> */
	public static final String BLACK_SQUARE_ICON = "black_square_icon";
	/** Constant <code>HOMOTYPIC_SYN_ICON="homotypic_syn_icon"</code> */
	public static final String HOMOTYPIC_SYN_ICON = "homotypic_syn_icon";
	/** Constant <code>HOMOTYPIC_SYN_ORIGINAL_ICON="homotypic_syn_original_icon"</code> */
	public static final String HOMOTYPIC_SYN_ORIGINAL_ICON = "homotypic_syn_original_icon";
	/** Constant <code>HETEROTYPIC_SYN_ICON="heterotypic_syn_icon"</code> */
	public static final String HETEROTYPIC_SYN_ICON = "heterotypic_syn_icon";
	public static final String NOM_INVAL_NUD_SYN_ICON = "nom_inval_nud_syn_icon";
	public static final String HOMOTYPIC_SYNONYM_IN_HETEROTYPIC_GROUP_ICON = "homotypic_synonym_in_heterotypic_group_icon";
	/** Constant <code>HETEROTYPIC_SYN_ORIGINAL_ICON="heterotypic_syn_original_icon"</code> */
	public static final String HETEROTYPIC_SYN_ORIGINAL_ICON = "heterotypic_syn_original_icon";
	/** Constant <code>MISAPPLIED_NAME_ICON="misapplied_icon"</code> */
	public static final String MISAPPLIED_NAME_ICON = "misapplied_icon";
	/** Constant <code>CONCEPT_ICON="concept_icon"</code> */
	public static final String CONCEPT_ICON = "concept_icon";
	/** Constant <code>AUTONYM_ICON="autonym_icon"</code> */
	public static final String AUTONYM_ICON = "autonym_icon";
	/** Constant <code>BASIONYM_ICON="basionym_icon"</code> */
	public static final String BASIONYM_ICON = "basionym_icon";
	/** Constant <code>ORTHOGRAPHIC_VARIANT_ICON="orthographic_variants_icon"</code> */
	public static final String ORTHOGRAPHIC_VARIANT_ICON = "orthographic_variants_icon";
	/** Constant <code>DB_ICON="db_icon"</code> */
	public static final String DB_ICON = "db_icon";
	/** Constant <code>MOVE_ICON="move_icon"</code> */
	public static final String MOVE_ICON = "move_icon";
	/** Constant <code>ACTIVE_DELETE_ICON="active_delete_icon"</code> */
	public static final String ACTIVE_DELETE_ICON = "active_delete_icon";
	/** Constant <code>SYNONYM_TO_TAXON_ICON="synoynm_to_taxon_icon"</code> */
	public static final String SYNONYM_TO_TAXON_ICON = "synoynm_to_taxon_icon";
	/** Constant <code>OPEN_TAXON_ICON="open_taxon_icon"</code> */
	public static final String OPEN_TAXON_ICON = "open_taxon_icon";
	/** Constant <code>ADD_CHILD_TAXON_ICON="add_taxon_icon"</code> */
	public static final String ADD_CHILD_TAXON_ICON = "add_taxon_icon";
	/** Constant <code>SWAP_SYNONYM_AND_TAXON_ICON="swap_syn_and_tax_icon"</code> */
	public static final String SWAP_SYNONYM_AND_TAXON_ICON = "swap_syn_and_tax_icon";
	/** Constant <code>QUICK_ADD_ICON="quick_add_icon"</code> */
	public static final String QUICK_ADD_ICON = "quick_add_icon";
	/** Constant <code>TAXON_TO_SYNONYM_ICON="tax_to_syn_icon"</code> */
	public static final String TAXON_TO_SYNONYM_ICON = "tax_to_syn_icon";
	/** Constant <code>ERROR_ANNOTATION_ICON="error_annotation_icon"</code> */
	public static final String ERROR_ANNOTATION_ICON = "error_annotation_icon";
	/** Constant <code>EDIT_BITMAP_ICON="edit_bitmap_icon"</code> */
	public static final String EDIT_BITMAP_ICON = "edit_bitmap_icon";
	/** Constant <code>IMG_DATASOURCE_CONNECTED="IMG_DATASOURCE_CONNECTED"</code> */
	public static final String IMG_DATASOURCE_CONNECTED = "IMG_DATASOURCE_CONNECTED";
	/** Constant <code>IMG_DATASOURCE_DISCONNECTED="IMG_DATASOURCE_DISCONNECTED"</code> */
	public static final String IMG_DATASOURCE_DISCONNECTED = "IMG_DATASOURCE_DISCONNECTED";
	/** Constant <code>IMG_NOT_FOUND="IMG_NOT_FOUND"</code> */
	public static final String IMG_NOT_FOUND = "IMG_NOT_FOUND";
	/** Constant <code>WARNING_ANNOTATION_ICON="warning"</code> */
	public static final String WARNING_ANNOTATION_ICON = "warning";
	/** Constant <code>THREE_PRONGED_EQUAL="3_pronged_equal"</code> */
	public static final String THREE_PRONGED_EQUAL = "3_pronged_equal";
	/** Constant <code>TWO_PRONGED_EQUAL="2_pronged_equal"</code> */
	public static final String TWO_PRONGED_EQUAL = "2_pronged_equal";
	/** Constant <code>MISAPPLIED_NAME="misapplied_name"</code> */
	public static final String MISAPPLIED_NAME = "misapplied_name";
	/** Constant <code>ORPHANED_TAXON="orphaned_taxon"</code> */
	public static final String ORPHANED_TAXON = "orphaned_taxon";
	/** Constant <code>EXPAND_ALL="expand_all"</code> */
	public static final String EXPAND_ALL = "expand_all";
	/** Constant <code>COLLAPSE_ALL="collapse_all"</code> */
	public static final String COLLAPSE_ALL = "collapse_all";
	

	
	/***************************************************************************
	 * IMAGE REGISTRY
	 **************************************************************************/
	private ImageRegistry imageRegistry;
	
	/**
	 * <p>getImageDescriptor</p>
	 *
	 * @param key a {@link java.lang.String} object.
	 * @return a {@link org.eclipse.jface.resource.ImageDescriptor} object.
	 */
	public static ImageDescriptor getImageDescriptor(String key) {
		return getInstance().getImageRegistry().getDescriptor(key);
	}

	/**
	 * <p>getImage</p>
	 *
	 * @param key a {@link java.lang.String} object.
	 * @return a {@link org.eclipse.swt.graphics.Image} object.
	 */
	public static Image getImage(String key) {
		return getInstance().getImageRegistry().get(key);
	}
	
	/**
	 * Gets an image for the given URL. Adds the image to the registry as needed.
	 *
	 * @param uri a {@link java.net.URI} object.
	 * @return a {@link org.eclipse.swt.graphics.Image} object.
	 */
	public static Image getImage(URI uri) {		
		
		String key = uri.toString();
		ImageRegistry registry = getInstance().getImageRegistry();

		Image image = registry.get(key);
		
		if (image == null) {
			ImageDescriptor desc;
			try {
				desc = ImageDescriptor.createFromURL(new URL(key));
				registry.put(key, desc);
				image = registry.get(key);
			} catch (DeviceResourceException e) {
				return getImageNotFound();
			} catch (MalformedURLException e) {
				// should never happen
			} catch (Exception e) {
				StoreUtil.error(ImageResources.class, e);
			}
			
//			logger.warn(ImageDescriptor.getMissingImageDescriptor().getClass());
//			logger.warn(desc.getClass());
//			
//			if (ImageDescriptor.getMissingImageDescriptor().equals(desc)) {
//				logger.warn("img not found");
//			}
			
		}
		
		return image;
	}
	
	/**
	 * <p>getImageNotFound</p>
	 *
	 * @return a {@link org.eclipse.swt.graphics.Image} object.
	 */
	public static Image getImageNotFound() {
		return getImage(ImageResources.IMG_NOT_FOUND);
	}

	/**
	 * <p>Getter for the field <code>imageRegistry</code>.</p>
	 *
	 * @return a {@link org.eclipse.jface.resource.ImageRegistry} object.
	 */
	public ImageRegistry getImageRegistry() {
        if (imageRegistry == null) {
            imageRegistry = TaxeditorStorePlugin.getDefault().createImageRegistry();
            initializeImageRegistry(imageRegistry);
        }
        return imageRegistry;
    }

	/**
	 * <p>initializeImageRegistry</p>
	 *
	 * @param registry a {@link org.eclipse.jface.resource.ImageRegistry} object.
	 */
	protected void initializeImageRegistry(ImageRegistry registry) {
		registerImage(registry, ADD_ICON, 
				"add_no_bg.gif");
		registerImage(registry, TRASH_ICON, 
				"trash.gif");
		registerImage(registry, EDIT_ICON, 
				"text.gif");
		registerImage(registry, EDIT_LOGO_ICON, 
				"edit_16x16.ico");
		registerImage(registry, WARNING_ICON,
				"warn_tsk.gif");
		registerImage(registry, BLACK_SQUARE_ICON,
				"accepted_small.gif");
		registerImage(registry, HOMOTYPIC_SYN_ICON,
				"homosyn_no_bg.gif");
		registerImage(registry,	HOMOTYPIC_SYN_ORIGINAL_ICON,
				"homosyn_original_no_bg.gif");
		registerImage(registry, HETEROTYPIC_SYN_ICON,
				"heterosyn_no_bg.gif");
		registerImage(registry, NOM_INVAL_NUD_SYN_ICON, 
				"nominval_nud_no_bg.gif");
		registerImage(registry,	HETEROTYPIC_SYN_ORIGINAL_ICON,
				"heterosyn_original_no_bg.gif");
		registerImage(registry, HOMOTYPIC_SYNONYM_IN_HETEROTYPIC_GROUP_ICON, 
				"homotypic_syn_hetero_group.gif");
		registerImage(registry, MISAPPLIED_NAME_ICON,
				"misapplied_no_bg.gif");
		registerImage(registry, CONCEPT_ICON,
				"concept_no_bg.gif");
		registerImage(registry, AUTONYM_ICON,
				"autonym_no_bg.gif");
		registerImage(registry, BASIONYM_ICON,
				"basionym_no_bg.gif");
		registerImage(registry, ORTHOGRAPHIC_VARIANT_ICON,
				"orthovariant_no_bg.gif");
		registerImage(registry, DB_ICON, 
				"db.gif");
		registerImage(registry, MOVE_ICON,
				"correction_change.gif");
		registerImage(registry, ACTIVE_DELETE_ICON,
				"delete_edit.gif");
		registerImage(registry, SYNONYM_TO_TAXON_ICON,
				"change.gif");
		registerImage(registry, OPEN_TAXON_ICON, 
				"open.gif");
		registerImage(registry, ADD_CHILD_TAXON_ICON,
				"new_child.gif");
		registerImage(registry,
				SWAP_SYNONYM_AND_TAXON_ICON, "swap2.gif");
		registerImage(registry, QUICK_ADD_ICON,
				"quick_add.gif");
		registerImage(registry, TAXON_TO_SYNONYM_ICON,
				"tax_to_syn.gif");
		registerImage(registry, ERROR_ANNOTATION_ICON,
				"error_co.gif");
		registerImage(registry, EDIT_BITMAP_ICON,
				"256color_16x16.bmp");
		registerImage(registry, IMG_DATASOURCE_CONNECTED, 
				"prj_obj.gif");
		registerImage(registry, IMG_DATASOURCE_DISCONNECTED, 
				"cprj_obj.gif");
		registerImage(registry, IMG_NOT_FOUND, 
				"pic_not_found.PNG");
		registerImage(registry, WARNING_ANNOTATION_ICON, 
				"warning_co.gif");
		registerImage(registry, THREE_PRONGED_EQUAL, 
				"3_pronged_equal_small.GIF");
		registerImage(registry, TWO_PRONGED_EQUAL, 
				"2_pronged_equal_small.GIF");		
		registerImage(registry, MISAPPLIED_NAME, 
				"ma_12x12.gif");	
		registerImage(registry, ORPHANED_TAXON, 
				"orphaned.gif");	
		registerImage(registry, EXPAND_ALL, 
				"expandAll.jpg");
		registerImage(registry, COLLAPSE_ALL, 
				"collapseall.gif");
		registerImage(registry, BROWSE_ICON, 
				"prj_obj.gif");
	}

	private void registerImage(ImageRegistry registry, String key,
			String fileName) {
		try {
			IPath path = new Path("icons/" + fileName); //$NON-NLS-1$
			URL url = FileLocator.find(TaxeditorStorePlugin.getDefault().getBundle(), path, null);
			if (url != null) {
				ImageDescriptor desc = ImageDescriptor.createFromURL(url);
				registry.put(key, desc);
			}
		} catch (Exception e) {
		}
	}
}
