// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;

import org.eclipse.ui.IMemento;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.XMLMemento;

import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>MementoHelper class.</p>
 *
 * @author n.hoffmann
 * @created Mar 29, 2010
 * @version 1.0
 */
public class MementoHelper {
	
	/**
	 * <p>readMementoFromFile</p>
	 *
	 * @param stateFile a {@link java.io.File} object.
	 * @return a org.eclipse.ui.IMemento object.
	 * @throws java.io.FileNotFoundException if any.
	 */
	public static IMemento readMementoFromFile(File stateFile) throws FileNotFoundException{
		FileInputStream input;
		try {
			input = new FileInputStream(stateFile);
			BufferedReader reader = new BufferedReader(
					new InputStreamReader(input, "utf-8")); //$NON-NLS-1$
			return XMLMemento.createReadRoot(reader);
		} catch (WorkbenchException e) {
			StoreUtil.error(MementoHelper.class, "Exception while reading the memento", e);
		} catch (UnsupportedEncodingException e) {
			StoreUtil.error(MementoHelper.class, "Exception while reading the memento", e);
		}

		return null;
	}
	
	/*
	 * Save the workbench UI in a persistence file.
	 */
	/**
	 * <p>saveMementoToFile</p>
	 *
	 * @param memento a org.eclipse.ui.IMemento object.
	 * @param stateFile a {@link java.io.File} object.
	 * @return a org.eclipse.ui.IMemento object.
	 */
	public static IMemento saveMementoToFile(IMemento memento, File stateFile) {
		if (stateFile == null || memento == null) {
			return null;
		}
		try {
			FileOutputStream stream = new FileOutputStream(stateFile);
			OutputStreamWriter writer = new OutputStreamWriter(stream, "utf-8"); //$NON-NLS-1$
			
			((XMLMemento)memento).save(writer);
			
			writer.close();
		} catch (IOException e) {
			stateFile.delete();
			StoreUtil.error(MementoHelper.class, "Could not save datasource state", e);
			return null;
		}

		// Success !
		return memento;
	}	
}
