// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.preference;

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import eu.etaxonomy.taxeditor.featuretree.FeatureTreeEditorWizard;

/**
 * <p>
 * FeatureTreePreferences class.
 * </p>
 * 
 * @author n.hoffmann
 * @created Aug 5, 2010
 * @version 1.0
 */
public class FeatureTreePreferences extends PreferencePage implements
		IWorkbenchPreferencePage {

	/**
	 * <p>
	 * Constructor for FeatureTreePreferences.
	 * </p>
	 */
	public FeatureTreePreferences() {
	}

	/**
	 * <p>
	 * Constructor for FeatureTreePreferences.
	 * </p>
	 * 
	 * @param title
	 *            a {@link java.lang.String} object.
	 */
	public FeatureTreePreferences(String title) {
		super(title);
	}

	/**
	 * <p>
	 * Constructor for FeatureTreePreferences.
	 * </p>
	 * 
	 * @param title
	 *            a {@link java.lang.String} object.
	 * @param image
	 *            a {@link org.eclipse.jface.resource.ImageDescriptor} object.
	 */
	public FeatureTreePreferences(String title, ImageDescriptor image) {
		super(title, image);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	/** {@inheritDoc} */
	@Override
	public void init(IWorkbench workbench) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse
	 * .swt.widgets.Composite)
	 */
	/** {@inheritDoc} */
	@Override
	protected Control createContents(Composite parent) {

		Composite composite = new Composite(parent, SWT.NULL);
		composite.setLayout(new GridLayout());

		final Button button_openFeatureTree = new Button(composite, SWT.PUSH);
		button_openFeatureTree.setText("Open FeatureTree Editor");

		button_openFeatureTree.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				FeatureTreeEditorWizard featureTreeEditorDialog = new FeatureTreeEditorWizard();
				WizardDialog dialog = new WizardDialog(getShell(),
						featureTreeEditorDialog);

				dialog.open();
			}
		});

		return composite;
	}
}
