// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewDerivedUnitBaseWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>FilteredDerivedUnitSelectionDialog class.</p>
 *
 * @author n.hoffmann
 * @created Jun 17, 2010
 * @version 1.0
 */
public class DerivedUnitSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<DerivedUnitBase> {

	/**
	 * <p>select</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param unit a {@link eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase} object.
	 * @return a {@link eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase} object.
	 */
	public static DerivedUnitBase select(Shell shell, ConversationHolder conversation, DerivedUnitBase unit){
		DerivedUnitSelectionDialog dialog = new DerivedUnitSelectionDialog(shell, conversation,
				"Choose Unit", false, DerivedUnitSelectionDialog.class.getCanonicalName(), unit);
		return getSelectionFromDialog(dialog);
	}
	
	/**
	 * <p>Constructor for FilteredDerivedUnitSelectionDialog.</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param title a {@link java.lang.String} object.
	 * @param multi a boolean.
	 * @param settings a {@link java.lang.String} object.
	 * @param cdmObject a {@link eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase} object.
	 */
	protected DerivedUnitSelectionDialog(Shell shell, ConversationHolder conversation, String title,
			boolean multi, String settings, DerivedUnitBase cdmObject) {
		super(shell, conversation, title, multi, settings, cdmObject);
	}
	
	/** {@inheritDoc} */
	@Override
	protected DerivedUnitBase getPersistentObject(UUID uuid) {
		Object object = CdmStore.getService(IOccurrenceService.class).load(uuid);
		
		SpecimenOrObservationBase specimenObservationBase = (SpecimenOrObservationBase) HibernateProxyHelper.deproxy(object);
		
		if(specimenObservationBase instanceof DerivedUnitBase){	
			return (DerivedUnitBase) specimenObservationBase; 
		}
		StoreUtil.error(this.getClass(), "Selected unit is not a derived unit", null);
		return null;		
	}

	/** {@inheritDoc} */
	@Override
	protected void initModel() {
		model = CdmStore.getService(IOccurrenceService.class).getDerivedUnitBaseUuidAndTitleCache();
	}

	/** {@inheritDoc} */
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return new NewDerivedUnitBaseWizard();
	}

	/** {@inheritDoc} */
	@Override
	protected String getNewWizardLinkText() {
		return "Click link to create a new <A>Unit</A>.";
	}
}
