// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.dialog.selection;

import java.util.UUID;

import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.occurrence.FieldObservation;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.taxeditor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.newWizard.NewFieldObservationWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>FilteredFieldObservationSelectionDialog class.</p>
 *
 * @author n.hoffmann
 * @created Jun 17, 2010
 * @version 1.0
 */
public class FieldObservationSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<FieldObservation> {
	
	/**
	 * <p>select</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param observation a {@link eu.etaxonomy.cdm.model.occurrence.FieldObservation} object.
	 * @return a {@link eu.etaxonomy.cdm.model.occurrence.FieldObservation} object.
	 */
	public static FieldObservation select(Shell shell, ConversationHolder conversation, FieldObservation observation){
		FieldObservationSelectionDialog dialog = new FieldObservationSelectionDialog(shell, conversation,
				"Choose Derived Unit", false, FieldObservationSelectionDialog.class.getCanonicalName(), observation);
		return getSelectionFromDialog(dialog);
	}

	/**
	 * <p>Constructor for FilteredFieldObservationSelectionDialog.</p>
	 *
	 * @param shell a {@link org.eclipse.swt.widgets.Shell} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param title a {@link java.lang.String} object.
	 * @param multi a boolean.
	 * @param settings a {@link java.lang.String} object.
	 * @param cdmObject a {@link eu.etaxonomy.cdm.model.occurrence.FieldObservation} object.
	 */
	protected FieldObservationSelectionDialog(Shell shell, ConversationHolder conversation, 
			String title, boolean multi, String settings,
			FieldObservation cdmObject) {
		super(shell, conversation, title, multi, settings, cdmObject);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#getPersistentObject(java.util.UUID)
	 */
	/** {@inheritDoc} */
	@Override
	protected FieldObservation getPersistentObject(UUID uuid) {
		Object object = CdmStore.getService(IOccurrenceService.class).load(uuid);
		
		SpecimenOrObservationBase specimenObservationBase = (SpecimenOrObservationBase) HibernateProxyHelper.deproxy(object);
		
		if(specimenObservationBase instanceof FieldObservation){	
			return (FieldObservation) specimenObservationBase; 
		}
		StoreUtil.error(this.getClass(), "Selected object is not a field observation", null);
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.dialogs.filteredSelection.AbstractFilteredCdmResourceSelectionDialog#initModel()
	 */
	/** {@inheritDoc} */
	@Override
	protected void initModel() {
		model = CdmStore.getService(IOccurrenceService.class).getFieldObservationUuidAndTitleCache();
	}

	/** {@inheritDoc} */
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard(String parameter) {
		return new NewFieldObservationWizard();
	}

	/** {@inheritDoc} */
	@Override
	protected String getNewWizardLinkText() {
		return "Click link to create a new <A>Field Observation</A>.";
	}
}
