/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.group.grantedauthority;

import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.SWT;

import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.layout.GridData;

import eu.etaxonomy.cdm.model.common.GrantedAuthorityImpl;
import eu.etaxonomy.cdm.persistence.hibernate.permission.CdmAuthority;
import eu.etaxonomy.taxeditor.model.ImageResources;

import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;


/**
 * Row widget for editing a single CDM Authority
 * 
 * @author cmathew
 * @created Mar 28, 2013
 *
 */
public class CdmAuthorityRow extends Composite {

	private final FormToolkit toolkit = new FormToolkit(Display.getCurrent());
	private Label lblEntity;
	private Label lblUuid;
	private CRUDOperationChooser operationChooser;
	private Button btnDelete;
	
	private GrantedAuthorityImpl grantedAuthorityI;
	private Label lblDirtyFlag;
	
	private CdmAuthorityCompositeViewer cdmaModel;
	
	/**
	 * Create the composite, made up of 5 elements :
	 * 1. '*' or depending on whether the row is being edited 
	 * 2. Classname of entity (e.g. TAXONNODE) 
	 * 3. Uuid of entity 
	 * 4. CRUD operations edit widget (this is essentially 4 checkboxes to select the 4 possible operations) 
	 * 5. Delete button (to delete the row) 
	 * 
	 * @param parent
	 * @param style
	 */
	public CdmAuthorityRow(Composite parent, int style) {
		super(parent, SWT.NONE);
		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				toolkit.dispose();
			}
		});
		toolkit.adapt(this);
		toolkit.paintBordersFor(this);
		setLayout(new GridLayout(5, false));
		

		
		lblDirtyFlag = new Label(this, SWT.NONE);
		lblDirtyFlag.setAlignment(SWT.CENTER);
		GridData gd_lblDirtyFlag = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_lblDirtyFlag.widthHint = 15;
		lblDirtyFlag.setLayoutData(gd_lblDirtyFlag);
		toolkit.adapt(lblDirtyFlag, true, true);
		lblDirtyFlag.setText("*");
		
		lblEntity = new Label(this, SWT.BORDER);
		GridData gd_lblEntity = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_lblEntity.heightHint = 30;
		gd_lblEntity.widthHint = 200;
		lblEntity.setLayoutData(gd_lblEntity);
		lblEntity.setAlignment(SWT.CENTER);
		toolkit.adapt(lblEntity, true, true);
		lblEntity.setText("Entity");
		
		lblUuid = new Label(this, SWT.BORDER);
		GridData gd_lblUuid = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1);
		gd_lblUuid.heightHint = 30;
		gd_lblUuid.widthHint = 270;
		lblUuid.setLayoutData(gd_lblUuid);
		lblUuid.setAlignment(SWT.CENTER);
		toolkit.adapt(lblUuid, true, true);
		lblUuid.setText("Uuid");
		
		operationChooser = new CRUDOperationChooser(this, SWT.BORDER);
		operationChooser.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1));
		toolkit.adapt(operationChooser);
		toolkit.paintBordersFor(operationChooser);
		
		btnDelete = new Button(this, SWT.NONE);

		btnDelete.setImage(ImageResources.getImage(ImageResources.TRASH_ICON));
		btnDelete.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1));
		btnDelete.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent arg0) {
				if(cdmaModel != null) {
					cdmaModel.removeCdmAuthority(grantedAuthorityI);
				}
			}
		});		
		
		toolkit.adapt(btnDelete, true, true);

	}

	/**
	 * Sets the dirty flag for this row
	 * 
	 * @param isDirty
	 */
	public void setDirty(boolean isDirty) {
		if(isDirty) {
			lblDirtyFlag.setText("*");
		} else {
			lblDirtyFlag.setText(" ");
		}
	}
	
	/**
	 * Create a {@link CdmAuthority} from a {@link GrantedAuthorityImpl} and create a row from it.
	 * 
	 * @param cdmaModel
	 * @param grantedAuthorityI
	 * @param isDirty
	 */
	public void setRowCdmAuthority(CdmAuthorityCompositeViewer cdmaModel, GrantedAuthorityImpl grantedAuthorityI, boolean isDirty) {
		this.grantedAuthorityI = grantedAuthorityI;		
		this.cdmaModel = cdmaModel;
		
		try {
			CdmAuthority cdmAuthority = CdmAuthority.fromGrantedAuthority(grantedAuthorityI);

			setDirty(isDirty);

			String entityStr = (cdmAuthority.getPermissionClass() == null)?"":cdmAuthority.getPermissionClass().toString();
			lblEntity.setText(entityStr);
			String uuidStr = (cdmAuthority.getTargetUUID() == null)?"":cdmAuthority.getTargetUUID().toString();
			lblUuid.setText(uuidStr);

			operationChooser.setAuthority(cdmaModel, grantedAuthorityI, cdmAuthority);				
			
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

}
