// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.openurl;

import java.util.List;

import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.ext.openurl.OpenUrlReference;


/**
 * @author n.hoffmann
 * @created Jan 31, 2011
 * @version 1.0
 */
public class OpenUrlResultPage extends WizardPage {

	public static final String NAME = "OpenUrlResultPage"; 
	
	private TableViewer viewer;
	
	/**
	 * @param pageName
	 */
	protected OpenUrlResultPage() {
		super(NAME);
		setTitle("Choose from results");
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createControl(Composite parent) {
		setPageComplete(false);
		
		Composite container = getWizard().getFormFactory().createComposite(parent);
		
		container.setLayout(new GridLayout());
		container.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		viewer = new TableViewer(container);
		viewer.getControl().setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		
		viewer.setContentProvider(new OpenUrlContentProvider());
		viewer.setLabelProvider(new OpenUrlLabelProvider());
		
		List<OpenUrlReference> results = getWizard().getResult();
		
		if(results == null || results.isEmpty()){
			String nomenclatuaralCitation = getWizard().getReference().getNomenclaturalCitation(getWizard().getReferenceDetail());
			setMessage("No results for the given reference: " + nomenclatuaralCitation, WizardPage.WARNING);
		}
		viewer.setInput(results);
		
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				
				OpenUrlReference reference = (OpenUrlReference) selection.getFirstElement();
				
				getWizard().setOpenUrlReference(reference);
				
				
				((OpenUrlReferencePage) getNextPage()).displayImage();
				
				setPageComplete(true);
			}
		});
		
		setControl(container);
		
	}
	
	public OpenUrlSelectorWizard getWizard(){
		return (OpenUrlSelectorWizard) super.getWizard();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.WizardPage#canFlipToNextPage()
	 */
	@Override
	public boolean canFlipToNextPage() {
		return super.canFlipToNextPage();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.WizardPage#isPageComplete()
	 */
	@Override
	public boolean isPageComplete() {
		return super.isPageComplete();
	}

}
