// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.description;

import java.util.Collection;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.Scope;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * <p>ScopeSection class.</p>
 *
 * @author n.hoffmann
 * @created Mar 18, 2010
 * @version 1.0
 */
public class ScopeSection extends AbstractEntityCollectionSection<TaxonDescription, Scope> {

	/**
	 * <p>Constructor for ScopeSection.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param style a int.
	 */
	public ScopeSection(CdmFormFactory cdmFormFactory, ConversationHolder conversation, ICdmFormElement parentElement, int style) {
		super(cdmFormFactory, conversation, parentElement, "Scope", style);
	}
	
	/** {@inheritDoc} */
	@Override
	public void addElement(Scope element) {
		getEntity().addScope(element);
	}

	/** {@inheritDoc} */
	@Override
	public Scope createNewElement() {
		return Scope.NewInstance();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Scope> getCollection(TaxonDescription entity) {
		return entity.getScopes();
	}

	/** {@inheritDoc} */
	@Override
	public String getEmptyString() {
		return "No scopes yet.";
	}

	/** {@inheritDoc} */
	@Override
	protected String getTooltipString() {
		return "Create a new scope";
	}

	/** {@inheritDoc} */
	@Override
	public void removeElement(Scope element) {
		getEntity().removeScope(element);
	}
}
