// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.key;

import java.util.Collection;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.description.PolytomousKey;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TaxonBaseSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection;

/**
 * @author n.hoffmann
 * @created Mar 29, 2011
 * @version 1.0
 */
public class TaxonomicScopeSection extends
		AbstractEntityCollectionSection<PolytomousKey, Taxon> {

	/**
	 * @param formFactory
	 * @param conversation
	 * @param parentElement
	 * @param title
	 * @param style
	 */
	public TaxonomicScopeSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement,
			int style) {
		super(formFactory, conversation, parentElement, "Taxonomic Scope",
				style);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#
	 * getCollection(java.lang.Object)
	 */
	@Override
	public Collection<Taxon> getCollection(PolytomousKey entity) {
		return entity.getTaxonomicScope();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#
	 * createNewElement()
	 */
	@Override
	public Taxon createNewElement() {
		Taxon selection = TaxonBaseSelectionDialog.selectTaxon(getShell(),
				getConversationHolder(), null);

		return selection;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#addElement
	 * (java.lang.Object)
	 */
	@Override
	public void addElement(Taxon element) {
		getEntity().addTaxonomicScope(element);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#
	 * removeElement(java.lang.Object)
	 */
	@Override
	public void removeElement(Taxon element) {
		getEntity().removeTaxonomicScope(element);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#
	 * getEmptyString()
	 */
	@Override
	public String getEmptyString() {
		return "No taxonomic scope yet.";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractEntityCollectionSection#
	 * getTooltipString()
	 */
	@Override
	protected String getTooltipString() {
		return "Add a taxon to the scope of this key";
	}

}
