// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;

import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LabelElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.supplemental.AbstractReferencedEntityElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * <p>NameRelationshipDetailElement class.</p>
 *
 * @author n.hoffmann
 * @created Mar 18, 2010
 * @version 1.0
 */
public class NameRelationshipDetailElement extends AbstractReferencedEntityElement<NameRelationship>{

	private TextWithLabelElement text_ruleConsidered;
	private EntitySelectionElement selection_toName;
	private TermComboElement<NameRelationshipType> combo_relationshipType;
	private LabelElement label;

	/**
	 * <p>Constructor for NameRelationshipDetailElement.</p>
	 *
	 * @param cdmFormFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.AbstractFormSection} object.
	 * @param entity a {@link eu.etaxonomy.cdm.model.name.NameRelationship} object.
	 * @param removeListener a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param style a int.
	 */
	public NameRelationshipDetailElement(CdmFormFactory cdmFormFactory,
			AbstractFormSection formElement,
			NameRelationship entity, SelectionListener removeListener, int style) {
		super(cdmFormFactory, formElement, entity, removeListener, style);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite#createControls(org.eclipse.swt.widgets.Composite, int)
	 */
	/** {@inheritDoc} */
	@Override
	public void createControls(ICdmFormElement element, int style) {
		label = formFactory.createLabel(element, null);
		
		text_ruleConsidered = formFactory.createTextWithLabelElement(element, "Rule Considered", null, SWT.NULL);
		
		super.createControls(element, style);
	}
	
	/** {@inheritDoc} */
	@Override
	public void setEntity(NameRelationship entity) {
		String text = entity.getFromName().getTitleCache() + " is " + entity.getType().getLabel() + " " + entity.getToName().getTitleCache();
		
		label.setText(text);
		text_ruleConsidered.setText(entity.getRuleConsidered());
		
		super.setEntity(entity);		
	}
	
	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == combo_relationshipType){
			getEntity().setType(combo_relationshipType.getSelection());
		}
		else if(eventSource == selection_toName){
			StoreUtil.error(getClass(), "No API call available.", null);
		}
		else if(eventSource == text_ruleConsidered){
			getEntity().setRuleConsidered(text_ruleConsidered.getText());
		}
		else if(eventSource == selection_reference){
			getEntity().setCitation(selection_reference.getSelection());
		}
		else if(eventSource == text_referenceDetail){
			getEntity().setCitationMicroReference(text_referenceDetail.getText());
		}
	}
	
	/** {@inheritDoc} */
	@Override
	public void setBackground(Color color) {
		if(label != null)
			label.setBackground(color);
		super.setBackground(color);
	}

}
