// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.occurrence;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.model.description.Sex;
import eu.etaxonomy.cdm.model.description.Stage;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

/**
 * <p>FieldObservationDetailElement class.</p>
 *
 * @author n.hoffmann
 * @created Jun 17, 2010
 * @version 1.0
 */
public class FieldObservationDetailElement extends AbstractCdmDetailElement<DerivedUnitFacade> {
	
	private TextWithLabelElement text_ecology;

	private TextWithLabelElement text_plantDescription;

	private TextWithLabelElement text_fieldObjectDefinition;

	private TextWithLabelElement text_fieldNotes;

	private NumberWithLabelElement number_individualCount;

	private TermComboElement<Sex> combo_sex;

	private TermComboElement<Stage> combo_lifeStage;
	
	/**
	 * <p>Constructor for FieldObservationDetailElement.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 */
	public FieldObservationDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}
		
	/** {@inheritDoc} */
	@Override
	protected void createControls(ICdmFormElement formElement,
			DerivedUnitFacade entity, int style) {
		text_ecology = formFactory.createTextWithLabelElement(formElement, "Ecology", entity.getEcology(CdmStore.getDefaultLanguage()), style);
		text_plantDescription = formFactory.createTextWithLabelElement(formElement, "Plant Description", entity.getPlantDescription(CdmStore.getDefaultLanguage()), style);
		// Disable for now
//		text_fieldObjectDefinition = formFactory.createTextWithLabelElement(formElement, "Field Object Definition", entity.getFieldObjectDefinition(CdmStore.getDefaultLanguage()), style);
		text_fieldNotes = formFactory.createTextWithLabelElement(formElement, "Field Notes", entity.getFieldNotes(), style);
		
		number_individualCount = formFactory.createIntegerTextWithLabelElement(formElement, "Individual Count", entity.getIndividualCount(), style);
		combo_lifeStage = formFactory.createTermComboElement(Stage.class, formElement, "Life Stage", entity.getLifeStage(), style);
		combo_sex = formFactory.createTermComboElement(Sex.class, formElement, "Sex", entity.getSex(), style);
	}

	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if(eventSource == text_ecology){
			getEntity().setEcology(text_ecology.getText());
		}
		else if(eventSource == text_plantDescription){
			getEntity().setPlantDescription(text_plantDescription.getText());
		}
		else if(eventSource == text_fieldObjectDefinition){
			getEntity().addFieldObjectDefinition(text_fieldObjectDefinition.getText(), CdmStore.getDefaultLanguage());
		}
		else if(eventSource == text_fieldNotes){
			getEntity().setFieldNotes(text_fieldNotes.getText());
		}
		else if(eventSource == number_individualCount){
			getEntity().setIndividualCount(number_individualCount.getInteger());
		}
		else if(eventSource == combo_lifeStage){
			getEntity().setLifeStage(combo_lifeStage.getSelection());
		}
		else if(eventSource == combo_sex){
			getEntity().setSex(combo_sex.getSelection());
		}
	}
}
