// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.section.occurrence;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.occurrence.Collection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.LanguageStringWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.NumberWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.PointElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;
import eu.etaxonomy.taxeditor.ui.element.ToggleableTextElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author n.hoffmann
 * @created Dec 16, 2010
 * @version 1.0
 */
public class GeneralDetailElement extends
		AbstractCdmDetailElement<DerivedUnitFacade> {

	private ToggleableTextElement toggleableText_titleCache;

	private EntitySelectionElement<NamedArea> selection_country;
	private LanguageStringWithLabelElement languageText_locality;
	private PointElement element_point;
	private NumberWithLabelElement number_elevation;
	private TimePeriodElement element_date;
	private EntitySelectionElement<AgentBase> selection_collector;
	private TextWithLabelElement text_collectingNumber;
	private EntitySelectionElement<Collection> selection_collection;
	private TextWithLabelElement text_accessionNumber;

	/**
	 * @param formFactory
	 * @param formElement
	 */
	public GeneralDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls
	 * (eu.etaxonomy.taxeditor.forms.ICdmFormElement, java.lang.Object, int)
	 */
	@Override
	protected void createControls(ICdmFormElement formElement,
			DerivedUnitFacade entity, int style) {
		toggleableText_titleCache = formFactory.createToggleableTextField(
				formElement, "Title Cache", entity.getTitleCache(),
				entity.isProtectedTitleCache(), style);

		selection_country = formFactory.createSelectionElement(NamedArea.class, getConversationHolder(),
				formElement, "Country",
				entity.getCountry(), EntitySelectionElement.NOTHING, style);
		languageText_locality = formFactory
				.createLanguageStringWithLabelElement(formElement, "Locality",
						entity.getLocality(), style);		
		element_point = formFactory.createPointElement(formElement,
				entity.getExactLocation(), style);
		number_elevation = formFactory.createIntegerTextWithLabelElement(
				formElement, "Absolute Elevation (m)",
				entity.getAbsoluteElevation(), style);
		element_date = formFactory.createTimePeriodElement(formElement, "Date",
				entity.getGatheringPeriod(), style);
		selection_collector = formFactory
				.createSelectionElement(AgentBase.class,
						getConversationHolder(), formElement, "Collector",
						entity.getCollector(), EntitySelectionElement.ALL,
						style);
		text_collectingNumber = formFactory.createTextWithLabelElement(formElement,
				"Collecting number", entity.getFieldNumber(), style);
		selection_collection = formFactory
				.createSelectionElement(Collection.class,
						getConversationHolder(), formElement, "Collection",
						entity.getCollection(),
						EntitySelectionElement.ALL, style);
		text_accessionNumber = formFactory.createTextWithLabelElement(
				formElement, "Accession Number", entity.getAccessionNumber(),
				style);
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java
	 * .lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == toggleableText_titleCache) {
			getEntity().setTitleCache(toggleableText_titleCache.getText(),
					toggleableText_titleCache.getState());
		} else if (eventSource == selection_country) {
			getEntity().setCountry(selection_country.getSelection());
		} else if (eventSource == languageText_locality) {
			LanguageString locality = languageText_locality.getLanguageString();
			getEntity().setLocality(locality);
		} else if (eventSource == element_point) {
			getEntity().setExactLocation(element_point.getPoint());
		} else if (eventSource == number_elevation) {
			getEntity().setAbsoluteElevation(number_elevation.getInteger());
		} else if (eventSource == element_date) {
			getEntity().setGatheringPeriod(element_date.getTimePeriod());
		} else if (eventSource == selection_collector) {
			getEntity().setCollector(selection_collector.getSelection());
		} else if (eventSource == text_collectingNumber) {
			getEntity().setFieldNumber(text_collectingNumber.getText());
		} else if (eventSource == selection_collection) {
			getEntity().setCollection(selection_collection.getSelection());
		} else if (eventSource == text_accessionNumber) {
			getEntity().setAccessionNumber(text_accessionNumber.getText());
		}

		if (eventSource != toggleableText_titleCache) {
			toggleableText_titleCache.setText(getEntity().getTitleCache());
		}
	}

}
