/**
 * 
 */
package eu.etaxonomy.taxeditor.ui.section.user;

import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CheckboxElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TextWithLabelElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.selection.EntitySelectionElement;

/**
 * @author n.hoffmann
 * 
 */
public class UserDetailElement extends AbstractCdmDetailElement<User> {

	private TextWithLabelElement text_username;
	private TextWithLabelElement text_emailAdress;
	private EntitySelectionElement<Person> selection_person;
	private CheckboxElement checkbox_credentialsNonExpired;
	private CheckboxElement checkbox_enabled;
	private CheckboxElement checkbox_accountNonLocked;
	private CheckboxElement checkbox_accountNonExpired;

	public UserDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#createControls
	 * (eu.etaxonomy.taxeditor.forms.ICdmFormElement, java.lang.Object, int)
	 */
	@Override
	protected void createControls(ICdmFormElement formElement, User entity,
			int style) {
		text_username = formFactory.createTextWithLabelElement(formElement,
				"Username", entity.getUsername(), style);

		if (userIsAuthenticated() || CdmStore.getLoginManager().isAdmin()) {
			formFactory.createEditPasswordElement(
					formElement, "Change Password", getConversationHolder(),
					entity, style);
		}

		text_emailAdress = formFactory.createTextWithLabelElement(formElement,
				"Email Adress", entity.getEmailAddress(), style);

		selection_person = formFactory
				.createSelectionElement(Person.class,
						getConversationHolder(), 
						formElement, 
						"Person",
						entity.getPerson(), 
						EntitySelectionElement.EDITABLE | EntitySelectionElement.SELECTABLE,
						style);

		checkbox_accountNonExpired = formFactory.createCheckbox(formElement,
				"Account Non Expired", entity.isAccountNonExpired(), style);
		checkbox_accountNonLocked = formFactory.createCheckbox(formElement,
				"Account Non Locked", entity.isAccountNonLocked(), style);
		checkbox_enabled = formFactory.createCheckbox(formElement, "Enabled",
				entity.isEnabled(), style);
		checkbox_credentialsNonExpired = formFactory.createCheckbox(
				formElement, "Credentials Non Expired",
				entity.isCredentialsNonExpired(), style);
	}

	/**
	 * 
	 * @return
	 */
	private boolean userIsAuthenticated() {
		return getEntity().equals(
				CdmStore.getLoginManager().getAuthenticatedUser());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.etaxonomy.taxeditor.section.AbstractCdmDetailElement#handleEvent(java
	 * .lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == text_username) {
			getEntity().setUsername(text_username.getText());
		} else if (eventSource == text_emailAdress) {
			getEntity().setEmailAddress(text_emailAdress.getText());
		} else if (eventSource == selection_person) {
			getEntity().setPerson(selection_person.getSelection());
		} else if (eventSource == checkbox_accountNonExpired) {
			getEntity().setAccountNonExpired(
					checkbox_accountNonExpired.getSelection());
		} else if (eventSource == checkbox_accountNonLocked) {
			getEntity().setAccountNonLocked(
					checkbox_accountNonLocked.getSelection());
		} else if (eventSource == checkbox_enabled) {
			getEntity().setEnabled(checkbox_enabled.getSelection());
		} else if (eventSource == checkbox_credentialsNonExpired) {
			getEntity().setCredentialsNonExpired(
					checkbox_credentialsNonExpired.getSelection());
		}
	}

}
