/**
* Copyright (C) 2011 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.userecords;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageString;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.description.CategoricalData;
import eu.etaxonomy.cdm.model.description.Modifier;
import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.cdm.model.description.StateData;
import eu.etaxonomy.taxeditor.editor.UsageTermCollection;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.TermStore;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;

 
/**
 * The context manager mediates context start/stop and workbench shutdowns to all registered listeners.
 *
 * @author a.theys	
 * @created mar 13, 2012
 * @version 1.0
 */
public class UseRecordDetailElement extends
		AbstractCdmDetailElement<CategoricalData> implements SelectionListener {

	private static final Logger logger = Logger
			.getLogger(UseRecordDetailElement.class);

	private TermComboElement<State> combo_UseCategory;
	private TermComboElement<State> combo_UseSubCategory;
	private TermComboElement<Modifier> combo_PlantPart;
	private TermComboElement<Modifier> combo_Country;
	private TermComboElement<Modifier> combo_HumanGroup;
	private TermComboElement<Modifier> combo_EthnicGroup;

	public UseRecordDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}

	/** {@inheritDoc} */
	@Override
	protected void createControls(ICdmFormElement formElement,
			CategoricalData entity, int style) {

		combo_UseCategory = formFactory.createTermComboElement(State.class,	this, UsageTermCollection.useCategoryVocabularyLabel, null, style);
		combo_UseCategory.addSelectionListener(this);
		combo_UseSubCategory = formFactory.createTermComboElement(State.class, this, UsageTermCollection.useSubCategoryVocabularyLabel, null, style);
		combo_PlantPart = formFactory.createTermComboElement(Modifier.class, this, UsageTermCollection.plantPartLabel, null, style);
		combo_HumanGroup = formFactory.createTermComboElement(Modifier.class, this, UsageTermCollection.humanGroupLabel, null, style);
		combo_HumanGroup.addSelectionListener(this);
		combo_EthnicGroup = formFactory.createTermComboElement(Modifier.class, this, UsageTermCollection.ethnicGroupLabel, null, style);
		combo_Country	= formFactory.createTermComboElement(Modifier.class, this, UsageTermCollection.countryLabel, null, style);
		

		List<State> termsUseCategory = setUseCategoryComboTerms(TermStore.getTerms(State.class, null, false));
		combo_UseCategory.setTerms(termsUseCategory);
		combo_UseCategory.setSelection(stateSelection(UsageTermCollection.useCategoryVocabularyLabel));
		
		List<State> termsSubCategory = setUseCategoryComboTerms(TermStore.getTerms(State.class, null, false), combo_UseCategory.getSelection());
		
		if (termsSubCategory != null) {
			combo_UseSubCategory.setTerms(termsSubCategory);
			combo_UseSubCategory.setSelection(stateSelection(UsageTermCollection.useSubCategoryVocabularyLabel));
			combo_UseSubCategory.setEnabled(true);
		}
		else {
			combo_UseSubCategory.setEnabled(false);
		}
		
		List<Modifier> plantPartsToAdd = setModifierComboTerms(TermStore.getTerms(Modifier.class, null, false), UsageTermCollection.plantPartLabel);
		combo_PlantPart.setTerms(plantPartsToAdd);
		combo_PlantPart.setSelection(modifierSelection(UsageTermCollection.plantPartLabel));
		
		List<Modifier> countriesToAdd = setModifierComboTerms(TermStore.getTerms(Modifier.class, null, false), UsageTermCollection.countryLabel);
		combo_Country.setTerms(countriesToAdd);
		combo_Country.setSelection(modifierSelection(UsageTermCollection.countryLabel));
		
		List<Modifier> humanGroupToAdd = setModifierComboTerms(TermStore.getTerms(Modifier.class, null, false), UsageTermCollection.humanGroupLabel);
		combo_HumanGroup.setTerms(humanGroupToAdd);
		combo_HumanGroup.setSelection(modifierSelection(UsageTermCollection.humanGroupLabel));
		
		List<Modifier> ethnicGroupToAdd = setEthnicGroupComboTerms(TermStore.getTerms(Modifier.class, null, false), combo_HumanGroup.getSelection());
		if (ethnicGroupToAdd != null) {
			combo_EthnicGroup.setTerms(ethnicGroupToAdd);
			combo_EthnicGroup.setSelection(modifierSelection(UsageTermCollection.ethnicGroupLabel));
			combo_EthnicGroup.setEnabled(true);
		} else {
			combo_EthnicGroup.setEnabled(false);
		}

	}

	/**
	 * Returns the select modifier
	 * @param comboCategory
	 * @return
	 */
	private Modifier modifierSelection(String comboCategory) {
		if(!getEntity().getModifiers().isEmpty()) {
			for (Modifier modifier : getEntity().getModifiers()) {
				if(GetVocabularyType(modifier, comboCategory)) {
					return modifier;
				}
			}
		} else {
			return null;
		}
		return null;
		
	}
	
	/**
	 * Returns the selected state
	 * @param comboCategory
	 * @return
	 */
	private State stateSelection(String comboCategory) {
		if (!getEntity().getStates().isEmpty()) {
			for (StateData statedata : getEntity().getStates()) {
				if (statedata.getModifyingText().get(CdmStore.getDefaultLanguage()) != null) {
					String testString = statedata.getModifyingText().get(CdmStore.getDefaultLanguage()).getText();
					if(testString.equals(comboCategory)) {
						if(statedata.getState() !=null) {
							return statedata.getState();
						}
					}
				}				
			}
		}
		else {
			return null;
		}
		return null;
		
	}
	
	/**
	 * This function sets the combo terms (Besides EthnicGroup
	 * @param listOfTerms
	 * @param modType
	 * @return
	 */
	private List<Modifier> setModifierComboTerms(List<Modifier> listOfTerms, String modType) {
		List<Modifier> termsToAdd = new ArrayList<Modifier>();
		for (Modifier term : listOfTerms) {

			if ((term.getPartOf() != null) && (term.getPartOf().getTitleCache().equals(modType))) {
				termsToAdd.add(term);
			}
			else if ((term.getVocabulary() != null) && (term.getPartOf() == null) && (term.getVocabulary().getTitleCache().equals(modType))) {
				termsToAdd.add(term);
			}
		}
		
		return termsToAdd;
	}
	

	/**
	 * This function sets the Use Category combo term
	 * @param listOfTerms
	 * @return
	 */
	private List<State> setUseCategoryComboTerms(List<State> listOfTerms) {
		List<State> termsToAdd = new ArrayList<State>();
		for (State term : listOfTerms) {

			if ((term.getPartOf() != null) && (term.getPartOf().getTitleCache().equals(UsageTermCollection.useCategoryVocabularyLabel))) {
				termsToAdd.add(term);
			}
			else if ((term.getVocabulary() !=null) && (term.getPartOf() == null) && (term.getVocabulary().getTitleCache().equals(UsageTermCollection.useCategoryVocabularyLabel))) {
				termsToAdd.add(term);
			}
		}
		return termsToAdd;

	}

	/**
	 * This function sets the Use SubCategory combo term
	 * @param listOfTerms
	 * @param selectedUseCategory
	 * @return
	 */
	private List<State> setUseCategoryComboTerms(List<State> listOfTerms,
			State selectedUseCategory) {
		List<State> termsToAdd = new ArrayList<State>();
		if (combo_UseCategory.getSelection() != null) {
			
			for (State term : listOfTerms) {
				if ((term.getPartOf() != null) && (term.getPartOf().getTitleCache().equals(selectedUseCategory.getTitleCache()))) {
					termsToAdd.add(term);
				}
				else if ((term.getVocabulary() !=null) && (term.getVocabulary().getTitleCache().equals(selectedUseCategory.getTitleCache()))) {
					termsToAdd.add(term);
				}
	
			}
		}
		else if (!getEntity().getStates().isEmpty()) {
			for (StateData stateData : getEntity().getStates()) {
				if (stateData.getModifyingText().get(CdmStore.getDefaultLanguage()) != null) {
					if ((stateData.getState() != null) && (stateData.getModifyingText().get(CdmStore.getDefaultLanguage()).getText() == UsageTermCollection.useCategoryVocabularyLabel)) {
						for (State term : listOfTerms) {
							if ((term.getPartOf() != null) && (term.getPartOf().getTitleCache().equals(stateData.getState().getTitleCache()))) {
								termsToAdd.add(term);
							} else if ((term.getVocabulary() != null) && (term.getVocabulary().getTitleCache().equals(stateData.getState().getTitleCache()))) {
								termsToAdd.add(term);
							}
						}
						
					}
				}
			}
			
		}
			
		else {
			return null;
		}

		return termsToAdd;
	}
	
	
	/**
	 * This function sets the EthnicGroup combo term
	 * @param listOfTerms
	 * @param selectedHumangroup
	 * @return
	 */
	private List<Modifier> setEthnicGroupComboTerms(List<Modifier> listOfTerms,
			Modifier selectedHumangroup) {
		List<Modifier> termsToAdd = new ArrayList<Modifier>();
		if (combo_HumanGroup.getSelection() != null) {
			for (Modifier term : listOfTerms) {
	
				if ((term.getPartOf() != null) && (term.getPartOf().getTitleCache().equals(selectedHumangroup.getTitleCache()))) {
					termsToAdd.add(term);
				}
				else if ((term.getVocabulary() != null) && (term.getVocabulary().getTitleCache().equals(selectedHumangroup.getTitleCache()))) {
					termsToAdd.add(term);
				}
			}
		}
		else if (!getEntity().getModifiers().isEmpty()) {
			for (Modifier modifier : getEntity().getModifiers()) {
				if (modifier.getMarkers() != null) {
					for (Marker marker : modifier.getMarkers()) {
						if(marker.getMarkerType().getTitleCache().equals(UsageTermCollection.humanGroupLabel)) {
							for (Modifier term : listOfTerms) {
								if ((term.getPartOf() != null) && (term.getPartOf().getTitleCache().equals(modifier.getTitleCache()))) {
									termsToAdd.add(modifier);
								} else if ((term.getVocabulary() != null) && (term.getVocabulary().getTitleCache().equals(modifier.getTitleCache()))) {
									termsToAdd.add(modifier);
								}
							}
							
						}
					}
				}
			}
		} else {
			return null;
		}
		
		return termsToAdd;
	}

	
	/** {@inheritDoc} */
	@Override
	public void handleEvent(Object eventSource) {
		if (eventSource == combo_UseCategory) {
			if (combo_UseCategory.getSelection() != null) {
				boolean isChanged = false;
				boolean isUseCategoryChanged = false;
				if (!getEntity().getStates().isEmpty()) {
					for (StateData useCategoryStateDataCheck : getEntity().getStates()) {
						Map<Language, LanguageString> modText = useCategoryStateDataCheck.getModifyingText();
						if(modText.get(CdmStore.getDefaultLanguage()) != null && modText.get(CdmStore.getDefaultLanguage()).getText().equals(UsageTermCollection.useCategoryVocabularyLabel)) {
							if(!useCategoryStateDataCheck.getState().getTitleCache().equals(combo_UseCategory.getSelection().getTitleCache())) {
								isUseCategoryChanged = true;
							}
							
						}
					}
					ListIterator<StateData> itrExistingStates = getEntity().getStates().listIterator();
					while(itrExistingStates.hasNext()) {
						StateData existingStateData = itrExistingStates.next();
						Map<Language, LanguageString> modifyingText = existingStateData
								.getModifyingText();
						if (modifyingText.get(CdmStore.getDefaultLanguage()) != null) {
							if (modifyingText.get(CdmStore.getDefaultLanguage()).getText().equals(UsageTermCollection.useCategoryVocabularyLabel)) {
								isChanged = true;
								if(!existingStateData.getState().getTitleCache().equals(combo_UseCategory.getSelection().getTitleCache())) {
									itrExistingStates.remove();
									StateData stateData = StateData.NewInstance(combo_UseCategory.getSelection());
									stateData.setState(combo_UseCategory.getSelection());
									stateData.putModifyingText(CdmStore.getDefaultLanguage(),UsageTermCollection.useCategoryVocabularyLabel);
									itrExistingStates.add(stateData);
									
								}
							} else if (modifyingText.get(CdmStore.getDefaultLanguage()).getText().equals(UsageTermCollection.useSubCategoryVocabularyLabel) && isUseCategoryChanged == true) {
								itrExistingStates.remove();
							}
						}
					}
				}
				
				
				if (isChanged == false) {
					StateData stateData = StateData
							.NewInstance(combo_UseCategory.getSelection());
					stateData.setState(combo_UseCategory.getSelection());
					stateData.putModifyingText(CdmStore.getDefaultLanguage(), UsageTermCollection.useCategoryVocabularyLabel);
					getEntity().addState(stateData);
				}
				
				combo_UseSubCategory.setEnabled(true);
	
			}
			else {
				if (!getEntity().getStates().isEmpty()) {
					Iterator<StateData> itr = getEntity().getStates().iterator();
					while(itr.hasNext()) {
						StateData stateToRemove = itr.next();
						itr.remove();
					}
					List<State> termsSubCategory = new ArrayList<State>();
					State emptyState = State.NewInstance();
					termsSubCategory.add(emptyState);
					combo_UseSubCategory.setTerms(termsSubCategory);
					combo_UseSubCategory.setEnabled(false);
				}
			}
		}
			
		if (eventSource == combo_UseSubCategory) {
			if (combo_UseSubCategory.getSelection() != null) {
				boolean isChanged = false;
				if (!getEntity().getStates().isEmpty()) {
					ListIterator<StateData> itrExistingStates = getEntity().getStates().listIterator();
					while(itrExistingStates.hasNext()) {
						StateData existingStateData = itrExistingStates.next();
						Map<Language, LanguageString> modifyingText = existingStateData
								.getModifyingText();
						if (modifyingText.get(CdmStore.getDefaultLanguage()) != null) {
							if (modifyingText.get(CdmStore.getDefaultLanguage()).getText().equals(UsageTermCollection.useSubCategoryVocabularyLabel)) {
								isChanged = true;
								if(!existingStateData.getState().getTitleCache().equals(combo_UseSubCategory.getSelection().getTitleCache())) {
									itrExistingStates.remove();
									StateData stateData = StateData.NewInstance(combo_UseSubCategory.getSelection());
									stateData.setState(combo_UseSubCategory.getSelection());
									stateData.putModifyingText(CdmStore.getDefaultLanguage(),UsageTermCollection.useSubCategoryVocabularyLabel);
									itrExistingStates.add(stateData);
								}
							}
						}
					}
				}
				
				if (isChanged == false){
					StateData stateData = StateData
							.NewInstance(combo_UseSubCategory.getSelection());
					stateData.setState(combo_UseSubCategory.getSelection());
					stateData.putModifyingText(CdmStore.getDefaultLanguage(), UsageTermCollection.useSubCategoryVocabularyLabel);
					getEntity().addState(stateData);
				}
	
			} else {
				if (!getEntity().getStates().isEmpty()) {
					Iterator<StateData> itrStateData = getEntity().getStates().iterator();
					while(itrStateData.hasNext()) {
						StateData existingStateData = itrStateData.next();
						Map<Language, LanguageString> modifyingText = existingStateData
								.getModifyingText();

						if (modifyingText.get(CdmStore.getDefaultLanguage()) != null) {
							if (modifyingText.get(CdmStore.getDefaultLanguage()).getText() == UsageTermCollection.useSubCategoryVocabularyLabel) {
								itrStateData.remove();
							}
						}
					}
				}
			}
		}
		if (eventSource == combo_PlantPart) {
			if (combo_PlantPart.getSelection() != null) {
				Iterator<Modifier> itrExistingModifiers = getEntity().getModifiers().iterator();
				while (itrExistingModifiers.hasNext()) {
					Modifier modifier = itrExistingModifiers.next();
					if(GetVocabularyType(modifier, UsageTermCollection.plantPartLabel)) {
						itrExistingModifiers.remove();
					}
						
				}	
				Modifier plantPart = combo_PlantPart.getSelection();
				getEntity().addModifier(plantPart);
			} else {
				Set<Modifier> modifiers = getEntity().getModifiers();
				if(!modifiers.isEmpty()) {
					Iterator<Modifier> itr = modifiers.iterator();
					while (itr.hasNext()) {
						Modifier currentMod = itr.next();
						if(GetVocabularyType(currentMod, UsageTermCollection.plantPartLabel)) {
							itr.remove();
						}
					}
				}
				
			}
		}
		if (eventSource == combo_Country) {
			if (combo_Country.getSelection() != null) {
				Iterator<Modifier> itrExistingModifiers = getEntity().getModifiers().iterator();
				while (itrExistingModifiers.hasNext()) {
					Modifier modifier = itrExistingModifiers.next();
					if(GetVocabularyType(modifier, UsageTermCollection.countryLabel)) {
						itrExistingModifiers.remove();
					}
						
				}
				Modifier country = combo_Country.getSelection();
				getEntity().addModifier(country);
			} else {
				Set<Modifier> modifiers = getEntity().getModifiers();
				if(!modifiers.isEmpty()) {
					Iterator<Modifier> itr = modifiers.iterator();
					while (itr.hasNext()) {
						Modifier currentMod = itr.next();
						if(GetVocabularyType(currentMod, UsageTermCollection.countryLabel)) {
							itr.remove();
						}
					}
				}
			}
		}
		if (eventSource == combo_HumanGroup) {
			if (combo_HumanGroup.getSelection() != null) {
				boolean isHumanGroupChanged = true;
				for (Modifier modToCheck: getEntity().getModifiers()) {
					if(modToCheck.equals(combo_HumanGroup.getSelection()) && (GetVocabularyType(modToCheck, UsageTermCollection.humanGroupLabel))) {
						isHumanGroupChanged = false;
				}
				
					
				}
				Iterator<Modifier> itrExistingModifiers = getEntity().getModifiers().iterator();
				while (itrExistingModifiers.hasNext()) {
					Modifier modifier = itrExistingModifiers.next();
					if((GetVocabularyType(modifier, UsageTermCollection.humanGroupLabel) || GetVocabularyType(modifier, UsageTermCollection.ethnicGroupLabel)) && isHumanGroupChanged) {
						itrExistingModifiers.remove();
					}
						
				}	
				if(isHumanGroupChanged){
					Modifier humanGroup = combo_HumanGroup.getSelection();
					getEntity().addModifier(humanGroup);
				}
				combo_EthnicGroup.setEnabled(true);
			} else {
				Set<Modifier> modifiers = getEntity().getModifiers();
				if(!modifiers.isEmpty()) {
					Iterator<Modifier> itr = modifiers.iterator();
					while (itr.hasNext()) {
						Modifier currentMod = itr.next();
						if(GetVocabularyType(currentMod, UsageTermCollection.humanGroupLabel) || GetVocabularyType(currentMod, UsageTermCollection.ethnicGroupLabel)) {
							itr.remove();
						}
					}
				}
				List<Modifier> emptyListToResetComboBox = new ArrayList<Modifier>();
				Modifier emptyMod = Modifier.NewInstance();
				emptyListToResetComboBox.add(emptyMod);
				combo_EthnicGroup.setTerms(emptyListToResetComboBox);
				combo_EthnicGroup.setEnabled(false);
				
			}
		}
		if (eventSource == combo_EthnicGroup) {
			if (combo_EthnicGroup.getSelection() != null) {
				Iterator<Modifier> itrExistingModifiers = getEntity().getModifiers().iterator();
					while (itrExistingModifiers.hasNext()) {
						Modifier modifier = itrExistingModifiers.next();
						if(GetVocabularyType(modifier, combo_HumanGroup.getSelection().getTitleCache())) {
							itrExistingModifiers.remove();
						}
						
					}	
					Modifier ethnicGroup = combo_EthnicGroup.getSelection();
					getEntity().addModifier(ethnicGroup);
			} else {
				Set<Modifier> modifiers = getEntity().getModifiers();
				if(!modifiers.isEmpty()) {
					
					Iterator<Modifier> itr = modifiers.iterator();
					while (itr.hasNext()) {
						Modifier currentMod = itr.next();
						if(GetVocabularyType(currentMod, UsageTermCollection.ethnicGroupLabel)) {
							itr.remove();
						}
					}
					
				}
			}
		}
	}
	
	

	/**
	 * This function Checks the vocabulary type
	 * @param term
	 * @param vocabularyExpected
	 * @return
	 */
	private boolean GetVocabularyType(Modifier term, String vocabularyExpected) {
		if ((term.getPartOf() != null)  && (term.getPartOf().getTitleCache().equals(vocabularyExpected))) {
			return true;
		}
		else if ((term.getVocabulary() != null) && (term.getPartOf()==null) && (term.getVocabulary().getTitleCache().equals(vocabularyExpected))) {
			return true;
		}
		else if ((vocabularyExpected.equals(UsageTermCollection.ethnicGroupLabel))  && ((term.getPartOf() != null))) {
			return true;
		}
	return false;
	}
	
	

	/** {@inheritDoc} */
	@Override
	public void setEntity(CategoricalData entity) {
		super.setEntity(entity);
	}

	/** {@inheritDoc} */
	@Override
	public void widgetSelected(SelectionEvent e) {
		Object eventSource = e.getSource();
		for (Object object: combo_UseCategory.getControls()) {
			if (object == eventSource) {
				State selectedUseCategory = combo_UseCategory.getSelection();
				if (selectedUseCategory != null) {
					combo_UseSubCategory.setTerms(setUseCategoryComboTerms(TermStore.getTerms(State.class, null, false),selectedUseCategory));
				
					if (!getEntity().getStates().isEmpty()) {
						Iterator<StateData> itrExistingState = getEntity().getStates().iterator();
						while(itrExistingState.hasNext()) {
							StateData existingStateData = (StateData) itrExistingState.next();
							Map<Language, LanguageString> modifyingText = existingStateData.getModifyingText();
							if (modifyingText.get(CdmStore.getDefaultLanguage()) != null) {
								if (modifyingText.get(CdmStore.getDefaultLanguage())
										.getText().equals(UsageTermCollection.useSubCategoryVocabularyLabel)) {
									combo_UseSubCategory.setSelection(stateSelection(UsageTermCollection.useSubCategoryVocabularyLabel));
								}
							}
						}
					}
				}
			}
		}
		for (Object object: combo_HumanGroup.getControls()) {
			if (object == eventSource) {
				Modifier selectedHumanGroup = combo_HumanGroup.getSelection();
				if (selectedHumanGroup != null) {
					combo_EthnicGroup.setTerms(setEthnicGroupComboTerms(TermStore.getTerms(Modifier.class, null, false), selectedHumanGroup));
					if (!getEntity().getModifiers().isEmpty()) {
						Iterator<Modifier> iterModifiers = getEntity().getModifiers().iterator();
						while (iterModifiers.hasNext()) {
							Modifier existingModifier = iterModifiers.next();
							if(GetVocabularyType(existingModifier, UsageTermCollection.ethnicGroupLabel)) {
								combo_EthnicGroup.setSelection(modifierSelection(UsageTermCollection.ethnicGroupLabel));
							}
						}
					}
					
				}
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void widgetDefaultSelected(SelectionEvent e) {
		// TODO Auto-generated method stub

	}

}
