// $Id$
/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.vocabulary;

import java.net.URISyntaxException;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.Representation;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.location.NamedAreaLevel;
import eu.etaxonomy.cdm.model.location.NamedAreaType;
import eu.etaxonomy.taxeditor.ui.combo.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.TimePeriodElement;

/**
 * @author l.morris
 * @date 20 Dec 2011
 *
 */
public class NamedAreaDetailElement extends DefinedTermDetailElement<NamedArea> {

	// level
	// type
	private TermComboElement<NamedAreaType> combo_namedAreaType;
	private TermComboElement<NamedAreaLevel> combo_namedAreaLevel;
	// validPeriod
	private TimePeriodElement timePeriod_validPeriod;
	
	/**
	 * @param formFactory
	 * @param formElement
	 */
	public NamedAreaDetailElement(CdmFormFactory formFactory,
			ICdmFormElement formElement) {
		super(formFactory, formElement);
	}
	
	public void createControls(ICdmFormElement formElement, NamedArea entity, int style) {
		super.createControls(formElement, entity, style);
		
		timePeriod_validPeriod = formFactory.createTimePeriodElement(formElement, "Valid Period", getEntity().getValidPeriod(), style);
		combo_namedAreaType = formFactory.createTermComboElement(NamedAreaType.class, formElement, "Named Area Type", getEntity().getType(), style);
		combo_namedAreaLevel = formFactory.createTermComboElement(NamedAreaLevel.class, formElement, "Named Area Level", getEntity().getLevel(), style);
		
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement#handleEvent(java.lang.Object)
	 */
	@Override
	public void handleEvent(Object eventSource) {
		
		if (eventSource == text_label) {
			getEntity().setLabel(text_label.getText());
			getEntity().setTitleCache(null);
		} else if (eventSource == text_description) {
			
			getEntity().getRepresentation(Language.DEFAULT()).setText(text_description.getText());
			
		} else if (eventSource == uri_uri) {
			
			try {
				getEntity().setUri(uri_uri.getUri());
			} catch (URISyntaxException e) {
				
				e.printStackTrace();
			}
		} else if (eventSource == text_abbreviatedLabel) {
			getEntity().getRepresentation(Language.DEFAULT()).setAbbreviatedLabel(text_abbreviatedLabel.getText());
		} else if (eventSource == timePeriod_validPeriod) {
			getEntity().setValidPeriod(timePeriod_validPeriod.getTimePeriod());
		} else if (eventSource == combo_namedAreaType) {
			getEntity().setType(combo_namedAreaType.getSelection());
		} else if (eventSource == combo_namedAreaLevel) {
			getEntity().setLevel(combo_namedAreaLevel.getSelection());
		}
		
		
	}

	
}
