// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.view.datasource;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.common.CdmMetaData;
import eu.etaxonomy.cdm.model.common.CdmMetaData.MetaDataPropertyName;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>CdmMetaDataAwareDataSourceContainer class.</p>
 *
 * @author n.hoffmann
 * @created Sep 22, 2010
 * @version 1.0
 */
public class CdmMetaDataAwareDataSourceContainer {
	
	
	
	/** Constant <code>DEFAULT_ENTRY="-"</code> */
	public static final String DEFAULT_ENTRY = "-";
	
	private ICdmDataSource dataSource;
	
	private boolean running;
	
	private Map<MetaDataPropertyName, String> metaDataMap;
	
	/**
	 * <p>Constructor for CdmMetaDataAwareDataSourceContainer.</p>
	 *
	 * @param dataSource a {@link eu.etaxonomy.cdm.database.ICdmDataSource} object.
	 * @param view a {@link eu.etaxonomy.taxeditor.view.datasource.CdmDataSourceViewPart} object.
	 */
	public CdmMetaDataAwareDataSourceContainer(ICdmDataSource dataSource){
		this.dataSource = dataSource;
		
		metaDataMap = getDefaultMetaDataMap();
		
	}

	/**
	 * <p>Getter for the field <code>dataSource</code>.</p>
	 *
	 * @return the dataSource
	 */
	public ICdmDataSource getDataSource() {
		return dataSource;
	}
	
	public void getMetaDataFromDataSource(){
		try {
			running = dataSource.testConnection();
		} catch (Exception e) {
			running = false;
			for(MetaDataPropertyName metaDataPropertyName : MetaDataPropertyName.values()){
				metaDataMap.put(metaDataPropertyName, DEFAULT_ENTRY);					
			}
		}
		
		try {
			if(isRunning()){
			
				try {
					for(MetaDataPropertyName metaDataPropertyName : MetaDataPropertyName.values()){
						Object result = dataSource.getSingleValue(metaDataPropertyName.getSqlQuery());
						if(result != null){
							metaDataMap.put(metaDataPropertyName, (String) result);
						}						
					}		
					dataSource.closeOpenConnections();
				} catch (SQLException e) {
					StoreUtil.warn(this.getClass(), "SQLException when trying to access: " + dataSource.getName() + ". " + e.getMessage());
				} 
			}
		} catch (Exception e) {
			StoreUtil.error(this.getClass(), e);
		}
	}
	
	private Map<MetaDataPropertyName, String> getDefaultMetaDataMap(){
		Map<MetaDataPropertyName, String> resultMap = new HashMap<MetaDataPropertyName, String>(MetaDataPropertyName.values().length);
		
		for(MetaDataPropertyName metaDataPropertyName : MetaDataPropertyName.values()){
			resultMap.put(metaDataPropertyName, null);
		}
		
		return resultMap;
	}

	/**
	 * <p>Getter for the field <code>metaDataMap</code>.</p>
	 *
	 * @return the metaDataMap
	 */
	public Map<MetaDataPropertyName, String> getMetaDataMap() {
		return metaDataMap;
	}
	
	/**
	 * <p>isDataSourceCompatible</p>
	 *
	 * @return a boolean.
	 */
	public boolean isDataSourceCompatible(){
		String version = metaDataMap.get(MetaDataPropertyName.DB_SCHEMA_VERSION);
		return version == null || version.equals(DEFAULT_ENTRY) ? false : CdmMetaData.isDbSchemaVersionCompatible(version);
	}

	/**
	 * <p>getMetaData</p>
	 *
	 * @param metaDataPropertyName a {@link eu.etaxonomy.cdm.model.common.CdmMetaData.MetaDataPropertyName} object.
	 * @return a {@link java.lang.String} object.
	 */
	public String getMetaData(MetaDataPropertyName metaDataPropertyName){
		String property = metaDataMap.get(metaDataPropertyName);
		return property != null ? property : DEFAULT_ENTRY;
	}
	
	/**
	 * <p>isRunning</p>
	 *
	 * @return the running
	 */
	public boolean isRunning() {
		return running;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	/** {@inheritDoc} */
	@Override
	public String toString() {
		return dataSource.getName();
	}
}
