// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.view.datasource.handler;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.progress.IProgressConstants;

import eu.etaxonomy.cdm.database.update.CdmUpdater;
import eu.etaxonomy.taxeditor.model.CdmProgressMonitorAdapter;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;
import eu.etaxonomy.taxeditor.view.datasource.CdmDataSourceViewPart;
import eu.etaxonomy.taxeditor.view.datasource.CdmMetaDataAwareDataSourceContainer;

/**
 * <p>UpdateDataSourceHandler class.</p>
 *
 * @author n.hoffmann
 * @created Sep 22, 2010
 * @version 1.0
 */
public class UpdateDataSourceHandler extends AbstractDataSourceHandler {
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.datasource.handler.AbstractDataSourceHandler#specificExecute(org.eclipse.core.commands.ExecutionEvent)
	 */
	/** {@inheritDoc} */
	@Override
	public boolean specificExecute(ExecutionEvent event) {
		final CdmMetaDataAwareDataSourceContainer container = getSelectedDataSourceContainer(event);
		
		final Display display = Display.getCurrent();
		
		Job job = new Job("Updating datasource " + container.getDataSource().getName()){

			@Override
			protected IStatus run(IProgressMonitor monitor) {
				CdmUpdater updater = new CdmUpdater();
				IStatus status = null;
				try{
					if(updater.updateToCurrentVersion(container.getDataSource(), CdmProgressMonitorAdapter.CreateMonitor(monitor))){
						display.asyncExec(new Runnable(){

							@Override
							public void run() {
								
								CdmDataSourceViewPart view = (CdmDataSourceViewPart) StoreUtil.getView(CdmDataSourceViewPart.ID, false);
								if(view != null){
									view.getViewer().update(new CdmMetaDataAwareDataSourceContainer[]{container}, null);
								}
								
							}
						});
						status = Status.OK_STATUS;
					}
				}catch(Exception e){
					status = new Status(IStatus.ERROR, TaxeditorStorePlugin.PLUGIN_ID, e.getMessage(), e);
					StoreUtil.errorDialog("Could not run updater", getClass(), status);
				}
				
				
				return status;
			}
			
		};
		
		job.setPriority(Job.BUILD);
		job.setProperty(IProgressConstants.KEEP_PROPERTY, true);
		job.schedule();
		
		return false;
	}
}
